/************************************************************************/
/*                       LICENSE AND DISCLAIMER					                */
/*                                                              		    */
/* This material contains IBM copyrighted sample programming source	    */
/* code for your consideration.  This sample code has not been		      */
/* thoroughly tested under all conditions.  IBM, therefore, cannot	    */
/* guarantee or imply reliability, serviceability, or function.		      */
/* IBM provides no program services for this material. This material	  */
/* is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR	*/
/* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	  */
/* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME		      */
/* JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO	  */
/* THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM	    */
/* BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER  	*/
/* CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,		    */
/* WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS    */
/* OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR	    */
/* OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH	    */
/* DAMAGES.


                          COPYRIGHT                              		    */
/*                              ---------                              	*/
/*               (C) Copyright IBM CORP. 1998, 1999                   	*/
/*               All rights reserved.                                  	*/
/*               US Government Users Restricted Rights -               	*/
/*               Use, duplication or disclosure restricted             	*/
/*               by GSA ADP Schedule Contract with IBM Corp.           	*/
/*               Licensed Material - Property of IBM                   	*/
/*******************************************************************
/*******************************************************************
*
* Module:
*   DBSMDSPS.C
*
* Purpose:
*      Utility routines for the DB sample programs to use.
*
*******************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "CWBDB.H"
#include "CWBSV.H"
#include "DBSMDSPS.H"
#include "CWBNL.H"
#include "CWBNLCNV.H" 

/****************************************************************************/
/* Routine to convert data in AS/400 ZONED format into PC double precision  */
/* floating point format.                                                   */
/****************************************************************************/
double cvt_zoned(
char *zonedata,                         // Character array representing num.
short zonescale,                        // Scale of input number
short width)                            // Source data width of zonedata
{
 short bi;
 double rdouble;                        // Our result

 rdouble = 0.0;

 for (bi=0; bi < width; bi++)
  {
   rdouble *=10.0;
   rdouble += (double)(((unsigned char)zonedata[bi]) & 0x0F);
   if ((((unsigned char)zonedata[width-1]) & 0xF0) == 0x70) // Negative?
       rdouble = -rdouble;
  }

 for (bi=1; bi<=zonescale; bi++)        // Adjust for scale factor, if any
   rdouble = rdouble/10.0;

 return rdouble;
} // end cvt_zoned


/****************************************************************************/
/* Routine to convert data in AS/400 PACKED format into PC double precision */
/* floating point format.                                                   */
/****************************************************************************/

double cvt_packed(
char *decdata,                          // Character array representing num.
short decscale,                         // Scale factor of number
short width)                            // Source data width of decdata
{
 short bi;
 double rdouble;                        // Return value

 rdouble = 0.0;

 for (bi=0; bi < width; bi++)
  {
   rdouble = rdouble*10.0;
   rdouble += (double)(((unsigned char)decdata[bi]) >> 4); //Cvt 1st half byte
   if (bi != (width-1))                 // Last input byte?
    {                                   // No, do 2nd half byte
     rdouble *=10.0;
     rdouble += (double)(((unsigned char)decdata[bi]) & 0x0F);
    }
   else                                 // Yes...
    {                                   // Determine sign digit
     if ((((unsigned char)decdata[bi]) & 0x0F) == 0x0D)
       rdouble = -rdouble;
    }
  }

 for (bi=1; bi<=decscale; bi++)         // Adjust for scale factor, if any
  rdouble = rdouble/10.0;

 return rdouble;
} /* cvt_packed */


/************************************************************************/
/* Display some error codes and call the routine to display the message */
/* text.                                                                */
/************************************************************************/
void displayErrors( char *            APIcall,
                    unsigned int      rcode,
                    unsigned short    errorClass,
                    signed long       errorCode,
                    cwbDB_DataHandle  msgID,
                    cwbDB_DataHandle  firstText,
                    cwbDB_DataHandle  secondText )
{

   printf("%s failed with return code %d.\n", APIcall, rcode );

   if ( rcode == 87 )
     printf("\tThis error means a parameter passed on the \n\t\t%s call was not valid.\n", APIcall );

   if ( rcode > CWBDB_START )
     displayClientError( rcode );   // Display the DB error message
   else
     if ( rcode >= CWB_USER_CANCELLED_COMMAND )
       displayBaseClientError( rcode );  // Display the Client Access message

   if ( rcode > CWBDB_DATASTREAM_TOO_SHORT )
   {
     printf("Server return code class: %d\n",errorClass);

     printf("      Server return code: %d\n",errorCode);

     displayMessages( msgID, firstText, secondText );
   }

   return;


} // end displayErrors


/**************************************************************************/
/* Display information returned from the AS/400 in the message ID,        */
/* first level message text, and second level message text data buffers.  */
/**************************************************************************/

void displayMessages( cwbDB_DataHandle  msgID,
                      cwbDB_DataHandle  firstText,
                      cwbDB_DataHandle  secondText )
{
   char                     *textBuffer;
   char                     tempBuffer[10];
//   char                     *tempPtr;
   char                     *textPtr;
   int                      i = 0, j = 0;
   unsigned long            textLen;
   cwbSV_ErrHandle          errMsgHandle;

   printf("Message info ----------------------------\n");

   /**************************************************/
   /* Process message ID.                            */
   /**************************************************/
   cwbDB_GetDataPointer( msgID, &textPtr, errMsgHandle );
   cwbDB_GetDataLength( msgID, &textLen, errMsgHandle  );
   if ( textLen > 0 )
   {
    textBuffer = (char*) malloc(textLen+2);
    memcpy( textBuffer, textPtr, textLen );
    textBuffer[textLen] = '\0';
    printf("   >>> Message ID: '%s' \n",textBuffer);
    if ( textBuffer != NULL )
    {
       free( textBuffer );
       textBuffer = NULL;
    }
   }
   else { printf("   >>> No message ID returned. \n"); }


   /**************************************************/
   /* Process first level message text.              */
   /**************************************************/
   cwbDB_GetDataPointer( firstText, &textPtr, errMsgHandle  );
   cwbDB_GetDataLength( firstText, &textLen, errMsgHandle  );
   if ( textLen > 0 )
   {
    textBuffer = (char*) malloc(textLen+2);
    memcpy( textBuffer, textPtr, textLen );
    textBuffer[textLen] = '\0';
    printf("   >>> 1st Level text: '%s' \n",textBuffer);
    if ( textBuffer != NULL )
    {
       free( textBuffer );
       textBuffer = NULL;
    }
   }
   else { printf("   >>> No first level message text returned. \n"); }


   /**************************************************/
   /* Process second level message text.             */
   /*   We'll get a little fancier here to pretty-up */
   /*   the results.                                 */
   /**************************************************/
   cwbDB_GetDataPointer( secondText, &textPtr, errMsgHandle  );
   cwbDB_GetDataLength( secondText, &textLen, errMsgHandle  );
   if ( textLen > 0 )
   {
    textBuffer = (char*) malloc(textLen+2);
    memcpy( textBuffer, textPtr, textLen );
    textBuffer[textLen] = '\0';
    textPtr = textBuffer;
    printf("   >>> 2nd Level text: \n");
    while ( *textPtr !=  '\0' )
    {
       memcpy(tempBuffer, textPtr, 10);
       j = strcspn( tempBuffer, " " );
       tempBuffer[j] = '\0';
       if ( strcmp(tempBuffer, "Recovery" ) == 0 )
       {
         printf("\n");
         i = 0;
       }
       if ( ( *textPtr == '-' ) && ( *(textPtr+1) == '-' ) )
       {
         printf("\n");
         i = 0;
       }
       if   ( i > 65 )
       {
         while ( ( *textPtr !=  '\0' ) && ( *textPtr !=  ' ' ) )
         {
           printf("%c", *textPtr );
           textPtr++;
         }
         textPtr++;
         // split it!
         printf("\n");
         i = 0;
       }
       printf("%c", *textPtr );
       textPtr++;
       i++;

    }
    printf("\n");
    if ( textBuffer != NULL )
    {
      free( textBuffer );
      textBuffer = NULL;
    }

   }
   else { printf("   >>> No second level message text returned. \n"); }

}



/************************************************************************/
/* Display some information about client errors which will not have     */
/* message text returned from the AS/400.                               */
/************************************************************************/
void displayClientError( unsigned int  clientRC )
{
 printf("Client error::  ");
 switch ( clientRC )
 {
    case CWBDB_CANNOT_CONTACT_SERVER              :
         printf("Cannot connect with the DB server\n" );
         break;
    case CWBDB_ATTRIBUTES_FAILURE                 :
         printf("Error processing server attributes\n" );
         break;
    case CWBDB_SERVER_ALREADY_STARTED             :
         printf("DB server already started\n" );
         break;
    case CWBDB_INVALID_DRDA_PKG_SIZE              :
         printf("DRDA package size not valid\n" );
         break;
    case CWBDB_REQUEST_MEMORY_ALLOCATION_FAILURE  :
         printf("Memory allocation failure  \n");
         break;
    case CWBDB_REQUEST_INVALID_CONVERSION         :
         printf("Conversion not valid       \n");
         break;
    case CWBDB_SERVER_NOT_ACTIVE                  :
         printf("DB server not active                  \n");
         break;
    case CWBDB_PARAMETER_ERROR                    :
         printf("Parameter error                          \n");
         break;
    case CWBDB_CLONE_CREATION_ERROR               :
         printf("Error creating duplicate handle          \n");
         break;
    case CWBDB_INVALID_DATA_FORMAT_FOR_CONNECTION :
         printf("Data format not valid for connection\n");
         break;
    case CWBDB_DATA_FORMAT_IN_USE                 :
         printf("Data format already in use               \n");
         break;
    case CWBDB_INVALID_DATA_FORMAT_FOR_DATA       :
         printf("Data not valid for this data format      \n");
         break;
    case CWBDB_STRING_ARG_TOO_LONG                :
         printf("String argument too long                 \n");
         break;
    case CWBDB_INVALID_INTERNAL_ARG               :
         printf("Internal argument error                  \n");
         break;
    case CWBDB_INVALID_NUMERIC_ARG                :
         printf("Numeric value not valid                  \n");
         break;
    case CWBDB_INVALID_ARG                        :
         printf("Argument not valid                       \n");
         break;
    case CWBDB_STMT_NOT_SELECT                    :
         printf("Statement is not a SELECT statement      \n");
         break;
    case CWBDB_STREAM_FETCH_NOT_COMPLETE          :
         printf("Stream fetch not complete          \n");
         break;
    case CWBDB_STREAM_FETCH_NOT_ACTIVE            :
         printf("Stream fetch not active            \n");
         break;
    case CWBDB_MISSING_DATA_PROCESSOR             :
         printf("Return data processor object missing      \n");
         break;
    case CWBDB_ILLEGAL_CLONE_REQUEST_TYPE         :
         printf("Can't do that!  \n");
         break;
    case CWBDB_UNSOLICITED_DATA                   :
         printf("Unexpected data from the server          \n");
         break;
    case CWBDB_MISSING_DATA                       :
         printf("Expected data from server missing        \n");
         break;
    case CWBDB_PARM_INVALID_BITSTREAM             :
         printf("Bitstream parameter not valid            \n");
         break;
    case CWBDB_CONSISTENCY_TOKEN_ERROR            :
         printf("Consistency token mismatch               \n");
         break;
    case CWBDB_INVALID_FUNCTION                   :
         printf("CWBDB_INVALID_FUNCTION                   \n");
         break;
    case CWBDB_FORMAT_INVALID_ARG                 :
         printf("CWBDB_FORMAT_INVALID_ARG                 \n");
         break;
    case CWBDB_INVALID_COLUMN_POSITION            :
         printf("CWBDB_INVALID_COLUMN_POSITION            \n");
         break;
    case CWBDB_INVALID_COLUMN_TYPE                :
         printf("CWBDB_INVALID_COLUMN_TYPE                \n");
         break;
    case CWBDB_ROW_VECTOR_NOT_EMPTY               :
         printf("CWBDB_ROW_VECTOR_NOT_EMPTY               \n");
         break;
    case CWBDB_ROW_VECTOR_EMPTY                   :
         printf("CWBDB_ROW_VECTOR_EMPTY                   \n");
         break;
    case CWBDB_MEMORY_ALLOCATION_FAILURE          :
         printf("CWBDB_MEMORY_ALLOCATION_FAILURE          \n");
         break;
    case CWBDB_INVALID_CONVERSION                 :
         printf("CWBDB_INVALID_CONVERSION                 \n");
         break;
    case CWBDB_DATASTREAM_TOO_SHORT               :
         printf("CWBDB_DATASTREAM_TOO_SHORT               \n");
         break;
    case CWBDB_SQL_WARNING                        :
         printf("CWBDB_SQL_WARNING                        \n");
         break;
    case CWBDB_SQL_ERROR                          :
         printf("CWBDB_SQL_ERROR                          \n");
         break;
    case CWBDB_SQL_PARAMETER_WARNING              :
         printf("CWBDB_SQL_PARAMETER_WARNING              \n");
         break;
    case CWBDB_SQL_PARAMETER_ERROR                :
         printf("CWBDB_SQL_PARAMETER_ERROR                \n");
         break;
    case CWBDB_LIST_SERVER_WARNING                :
         printf("CWBDB_LIST_SERVER_WARNING                \n");
         break;
    case CWBDB_LIST_SERVER_ERROR                  :
         printf("CWBDB_LIST_SERVER_ERROR                  \n");
         break;
    case CWBDB_LIST_PARAMETER_WARNING             :
         printf("CWBDB_LIST_PARAMETER_WARNING             \n");
         break;
    case CWBDB_LIST_PARAMETER_ERROR               :
         printf("CWBDB_LIST_PARAMETER_ERROR               \n");
         break;
    case CWBDB_NDB_FILE_SERVER_WARNING            :
         printf("CWBDB_NDB_FILE_SERVER_WARNING            \n");
         break;
    case CWBDB_NDB_FILE_SERVER_ERROR              :
         printf("CWBDB_NDB_FILE_SERVER_ERROR              \n");
         break;
    case CWBDB_FILE_PARAMETER_WARNING             :
         printf("CWBDB_FILE_PARAMETER_WARNING             \n");
         break;
    case CWBDB_FILE_PARAMETER_ERROR               :
         printf("CWBDB_FILE_PARAMETER_ERROR               \n");
         break;
    case CWBDB_GENERAL_SERVER_WARNING             :
         printf("CWBDB_GENERAL_SERVER_WARNING             \n");
         break;
    case CWBDB_GENERAL_SERVER_ERROR               :
         printf("CWBDB_GENERAL_SERVER_ERROR               \n");
         break;
    case CWBDB_EXIT_PROGRAM_WARNING               :
         printf("CWBDB_EXIT_PROGRAM_WARNING               \n");
         break;
    case CWBDB_EXIT_PROGRAM_ERROR                 :
         printf("CWBDB_EXIT_PROGRAM_ERROR                 \n");
         break;
    case CWBDB_DATA_BUFFER_TOO_SMALL              :
         printf("CWBDB_DATA_BUFFER_TOO_SMALL              \n");
         break;
    case CWBDB_NL_CONVERSION_ERROR                :
         printf("CWBDB_NL_CONVERSION_ERROR                \n");
         break;
    case CWBDB_COMMUNICATIONS_ERROR               :
         printf("CWBDB_COMMUNICATIONS_ERROR               \n");
         break;
    case CWBDB_INVALID_ARG_API                    :
         printf("CWBDB_INVALID_ARG_API                    \n");
         break;
    case CWBDB_MISSING_DATA_HANDLER               :
         printf("CWBDB_MISSING_DATA_HANDLER               \n");
         break;
    case CWBDB_REQUEST_DATASTREAM_NOT_VALID       :
         printf("CWBDB_REQUEST_DATASTREAM_NOT_VALID       \n");
         break;
    case CWBDB_SERVER_UNABLE                      :
         printf("CWBDB_SERVER_UNABLE                      \n");
         break;
    case CWBDB_LAST_STREAM_CHUNK                  :
         printf("CWBDB_LAST_STREAM_CHUNK                  \n");
         break;
    default : printf("UNKNOWN?????\n"); break;
  }
 }

/************************************************************************/
/* Display the data returned from the AS/400.                           */
/************************************************************************/
int displayData( cwbDB_DataHandle    data,
                  cwbDB_FormatHandle  dataFormat,
                  cwbDB_DataHandle    indicators,
                  cwbSV_ErrHandle     errHandle,	
				  cwbDB_DataHandle	sqlRequestHandle)	//Required for LOBs
{

// A structure to hold info about a column.
struct columnInfo
{
 unsigned long    columnLength      ;
 signed   short   columnClientType  ;
 signed   short   columnServerType  ;
 unsigned short   columnScale       ;
 unsigned short   columnCCSID       ;
 unsigned short   columnPrecision   ;
 unsigned long    columnNameLength  ;
};


 unsigned int     rc                = 0;
 unsigned int     i = 0,          k = 0;
 unsigned long    j = 0, parmLength = 0;
 unsigned long    nbrRows           = 0;
 unsigned long    rowSize           = 0;

 // Some assumptions are going to be made about the number of columns
 // and size of rows. A real application would probably want to dynamically
 // allocate some the buffers.
 char             tempBuffer[2512];
 char             headerLine[2512];
 char             *headerStringPtr;
 unsigned long    columnCount       = 0;
 unsigned long    dataLen           = 0;
 long             *pLong            = NULL;
 char             *dataPtr          = NULL;
 char             *indPtr           = NULL;
 char             *baseDataPtr      = NULL;
 char             *charPtr          = NULL;
 long             *longPtr          = NULL;
 short            *shortPtr         = NULL;
 float            *floatPtr         = NULL;
 double           *doublePtr        = NULL;
 double           doubleData        = 0   ;
 struct columnInfo       columnInf[ 2500 ];
 
 //Variables required for lob locators
 cwbDB_DataHandle	hLOBData;	// A DataHandle to receive lob data from a locator
 FILE				*stream;
 unsigned long		ulLOBLength;
 unsigned long		ulMaxLOBSize;
 UCHAR *			lpLOBData;
 unsigned long		nmbrerrs;	//cwbNL vars 
 unsigned long		posoferr;	//cwbNL vars
 unsigned long		rqdlen;		//cwbNL vars
 char				szBLOBFn[512];


 cwbDB_DataHandle	columnName; // A DB data object to receive column names
 cwbSV_ErrHandle	msgHandle;  // A service trace handle

   // Initialize the local buffers to nulls.
   memset( headerLine, 0, sizeof(headerLine) );
   memset( tempBuffer, 0, sizeof(tempBuffer) );

   // Need a data handle to get column name information from the data format
   //  (instantiate a DB data object)
   rc = cwbDB_CreateDataHandle( &columnName, msgHandle );
   if (rc != CWB_OK)
   {
       printf("\ncwbDB_CreateDataHandle (column name) failed with return ");
       printf("code %d.\n", rc);
       return(rc);
   }

   // get size of returned data
   cwbDB_GetDataLength( data, &dataLen, errHandle  );
   if ( dataLen == 0 )
   { // Hmmm. Thought we had some data....?
     printf(">>> NO DATA RETURNED\n");
     return(0);
   }

   // get a pointer to the indicator buffer
   //   (There is one indicator value for each column value
   //    of each row of data that is returned from the AS/400.
   //    The indicator will be a negative number if the value
   //    for the column is NULL.  If an error occurs while
   //    converting the data, a character 'E' will be placed
   //    in that column's indicator field.                     )
   cwbDB_GetDataPointer( indicators, &indPtr, errHandle );

   // Get pointer to data handle's buffer space. The data handle's buffer
   // may move around if it is reused on a call to get data from the AS/400.
   cwbDB_GetDataPointer( data, &dataPtr, errHandle );

   // This pointer will be good for the life of this procedure
   baseDataPtr = dataPtr;

   // How big is a row of data from the returned data?
   rc = cwbDB_GetRowSize( dataFormat, CWBDB_LOCAL, &rowSize, errHandle );
   if (rc != CWB_OK)
   {
       printf("\ncwbDB_GetRowSize failed ");
       printf("with return code %d.\n", rc);
       return(rc);
   }

   // Calculate the number of rows in the returned data buffer
   //   (divide the size of the data by the row size)
   nbrRows =  dataLen/rowSize;

   // Announce the results to the world
   printf("\nThere are %d rows in the data buffer\n\n", nbrRows );

   // Find out how many columns (fields) there are in each row of data.
   rc = cwbDB_GetColumnCount( dataFormat, &columnCount, errHandle );
   if (rc != CWB_OK)
   {
       printf("\ncwbDB_GetColumnCount failed on column %d");
       printf("with return code %d.\n", i, rc);
       return(rc);
   }

   // Announce the results to the world
//   printf("There are %d columns in each row of data\n", columnCount );

   // This loop is for generating a heading for the data.
   //   It isn't fancy, but it helps some....
   for ( i = 1; (unsigned int)i <= columnCount ; i++ )
   {
      /***********************************************************/
      /* Going through the data format column-by-column to get   */
      /* the information about the data.                         */
      /*     Want to know the length (width) of each column.     */
      /*     Want to know the (client) type of each column.      */
      /*     Want to know the (AS/400) type of each column.      */
      /*     Want to know the Coded Character Set ID (CCSID) of  */
      /*         the data.                                       */
      /*     Want to know the name of each column.               */
      /***********************************************************/

      // Get the length (width) of the column.
      rc = cwbDB_GetColumnLength( dataFormat,
                                  CWBDB_LOCAL,
                                  i,
                                  &(columnInf[i].columnLength),
                                  errHandle );
      if (rc != CWB_OK)
      {
          printf("\ncwbDB_GetColumnLength failed ");
          printf("with return code %d. Column %d\n", rc, i);
          return(rc);
      }

      // Get the PC type of the column.
      rc = cwbDB_GetColumnType( dataFormat,
                                CWBDB_LOCAL,
                                i,
                                &(columnInf[i].columnClientType),
                                errHandle );
      if (rc != CWB_OK)
      {
          printf("\ncwbDB_GetColumnType failed ");
          printf("with return code %d. Column %d\n", rc, i);
          return(rc);
      }

      // Get the AS/400 type of the column.
      rc = cwbDB_GetColumnType( dataFormat,
                                CWBDB_SYSTEM,
                                i,
                                &(columnInf[i].columnServerType),
                                errHandle );
      if (rc != CWB_OK)
      {
          printf("\ncwbDB_GetColumnType failed ");
          printf("with return code %d. Column %d\n", rc, i);
          return(rc);
      }

      // Get the CCSID (codepage, sort-of) of the column
      rc = cwbDB_GetColumnCCSID( dataFormat,
                                 CWBDB_SYSTEM,
                                 i,
                                 &(columnInf[i].columnCCSID),
                                 errHandle );
      if (rc != CWB_OK)
      {
          printf("\ncwbDB_GetColumnCCSID failed ");
          printf("with return code %d. Column %d\n", rc, i);
          return(rc);
      }

      // Get the name of the column
      rc = cwbDB_GetColumnName( dataFormat,
                                i,
                                columnName,
                                errHandle );
      if (rc != CWB_OK)
      {
          printf("\ncwbDB_GetColumnName failed ");
          printf("with return code %d. Column %d\n", rc, i);
          return(rc);
      }

      // Get a pointer to the buffer of the data object containing the
      // name of the column
      cwbDB_GetDataPointer( columnName, &headerStringPtr, errHandle );

      // Get the length of the column name
      cwbDB_GetDataLength( columnName,
                           &(columnInf[i].columnNameLength),
                           errHandle  );

      j = strlen(headerLine);
      // Next code is a kludge to make the header look a little better...
      if ( columnInf[i].columnNameLength < columnInf[i].columnLength )
      {
        memcpy(&headerLine[ j ], headerStringPtr, columnInf[i].columnNameLength );
        for ( k = 0; ((unsigned int)k <= columnInf[i].columnLength &&
						(unsigned int)k <= 100);	//dxd limit it so it doesn't overrun 
				k++ )
          headerLine[ j+columnInf[i].columnNameLength+k ] = ' ';
        headerLine[ j+columnInf[i].columnNameLength+k ] = '\0';
      }
      else
      {
        if ( columnInf[i].columnLength <= 5 )
        { // columnInf[i].columnLength = 15;
          memcpy(&headerLine[ j ], headerStringPtr, 15 );
          headerLine[ j+15 ] = '\0';
        }
        else
        {
          memcpy(&headerLine[ j ], headerStringPtr, columnInf[i].columnLength );
          headerLine[ j+columnInf[i].columnLength ] = '\0';
        }
      }

   }

   printf("\n%s\n", headerLine );

   // Now process the real data. Go through row-by-row.
   for ( j = 0; j < nbrRows; j++ )
   {
     // Calculate offset of next column's data
     dataPtr = baseDataPtr + ( j * rowSize );

     // Process the row data. Go through column-by-column.
     for ( i = 1; (unsigned int)i <= columnCount ; i++ )
     {

         switch ( columnInf[i].columnClientType )
         {
         case CWBDB_PCNOCONVERSION :
                                     break;
         case CWBDB_PCSTRING       :
                                     charPtr = dataPtr;
                                     memcpy(tempBuffer, charPtr, columnInf[i].columnLength);
                                     tempBuffer[columnInf[i].columnLength] = '\0';
                                     printf("'%s'",tempBuffer);
                                     break;
         case CWBDB_PCLONG         :
                                     longPtr = (long*)dataPtr;
                                     printf("%ld", *longPtr);
                                     if ( *longPtr < -2000 )
                                     {
                                       charPtr = dataPtr;
                                       memcpy(tempBuffer, charPtr, columnInf[i].columnLength);
                                       printf(" (");
                                       for ( k = 0; k < columnInf[i].columnLength; k++ )
                                         printf("%0X",tempBuffer[k]);
                                       printf(") ");
                                     }
                                     break;
         case CWBDB_PCSHORT        :
                                     shortPtr = (short*)dataPtr;
                                     printf("%d", *shortPtr);

                                     shortPtr = (short*)dataPtr;
                                     printf("%d", *shortPtr);
                                     if ( *shortPtr < -2000 )
                                     {
                                       charPtr = dataPtr;
                                       memcpy(tempBuffer, charPtr, columnInf[i].columnLength);
                                       printf(" (");
                                       for ( k = 0; k < columnInf[i].columnLength; k++ )
                                         printf("%0X",tempBuffer[k]);
                                       printf(") ");
                                     }
                                     break;
         case CWBDB_PCFLOAT        :
                                     floatPtr = (float*)dataPtr;
                                     printf("%f", *floatPtr );
                                     break;
         case CWBDB_PCDOUBLE       :
                                     doublePtr = (double*)dataPtr;
                                     printf("%f", *doublePtr );
                                     break;
         case CWBDB_PCPACKED       :
                                     charPtr = dataPtr;
                                     rc = cwbDB_GetColumnScale( dataFormat,
                                                                CWBDB_LOCAL,
                                                                i,
                                                                &(columnInf[i].columnScale),
                                                                errHandle );
                                     if (rc != CWB_OK)
                                     {
                                         printf("\ncwbDB_GetColumnScale");
                                         printf("failed with return code ");
                                         printf("%d. Column %d\n", rc, i);
                                         return(rc);
                                     }
                                     doubleData = cvt_packed( charPtr,
                                                              (columnInf[i].columnScale),
                                                              columnInf[i].columnLength );
                                     printf("%F", doubleData );

                                     break;
         case CWBDB_PCZONED        :
                                     charPtr = dataPtr;
                                     rc = cwbDB_GetColumnScale( dataFormat,
                                                                CWBDB_LOCAL,
                                                                i,
                                                                &columnInf[i].columnScale,
                                                                errHandle );
                                     if (rc != CWB_OK)
                                     {
                                         printf("\ncwbDB_GetColumnScale");
                                         printf("failed with return code ");
                                         printf("%d. Column %d\n", rc, i);
                                         return(rc);
                                     }
                                     doubleData = cvt_zoned(  charPtr,
                                                              columnInf[i].columnScale,
                                                              columnInf[i].columnLength );
                                     printf("%F", doubleData );

                                     break;
         case CWBDB_PCINVALIDTYPE  :
                                     break;
         case CWBDB_PCVARSTRING    :
                                     charPtr = dataPtr + 2;			//First two bytes are string length
                                     memcpy(tempBuffer, charPtr, *(short*)dataPtr);	
                                     tempBuffer[*(short*)dataPtr] = '\0';
                                     printf("'%s'",tempBuffer);
                                     break;
         case CWBDB_PCGRAPHIC      :  // OK, I got lazy on the DBCS stuff...
                                     break;
         case CWBDB_PCVARGRAPHIC   :
                                     break;
		 //****************************** LOB support
		case CWBDB_PCBLOBLOCATOR: 
		case CWBDB_PCCLOBLOCATOR: 
		//case CWBDB_PCDBCLOBLOCATOR: 			
			//Create a data handle that will be used to fetch the lob data
			rc = cwbDB_CreateDataHandle( &hLOBData, msgHandle ); // Data handle for all the LOB data
 		    if (rc != CWB_OK)
			{
				printf("\ncwbDB_CreateDataHandle (LOB Data) failed with return ");
				printf("code %d.\n", rc);
				return(rc);
			}
			
			// Column length is always reported as 4 (size of the locator itself).
			// so we must call this to get the Maximum size of the lob field.
			// (BLOB max size) = (CLOB max size) = (2 x DBCLOB max size) because DBCLOB in characters
			if (cwbDB_GetLOBMaxSize(dataFormat,
			                    i,			
			                    &ulMaxLOBSize,		//Update the column info
								msgHandle) != CWB_OK) 
			{
				printf("\ncwbDB_GetLOBMaxSize failed with return ");
				printf("code %d.\n", rc);
				return(rc);
			}
					
			rc = cwbDB_RetrieveLOBData(sqlRequestHandle, // Request handle for this storage object
					                   hLOBData,		 // Data handle (how we get actual data back)
					                   *(long*)dataPtr,  // LOB locator in AS/400 database
					                   ulMaxLOBSize,	 // Max size (get whole thing). We don't know
					                                     // the size, but using the max possible size
					                                     // will get the whole thing.
					                   0,				 // From the beginning of the LOB
					                   i,				 // Pass the column ordinal (this allows multiple rows of data with locators to be valid)
					                   msgHandle);		 // Error information
 		    if (rc != CWB_OK)
			{
				printf("\ncwbDB_RetrieveLOBData failed with return ");
				printf("code %d.\n", rc);
				return(rc);
			}
			
			rc = cwbDB_GetDataLength (hLOBData, &ulLOBLength, msgHandle);
 		    if (rc != CWB_OK)
			{
				printf("\ncwbDB_GetDataLength failed with return ");
				printf("code %d.\n", rc);
				return(rc);
			}

			rc = cwbDB_GetDataPointer(hLOBData, &lpLOBData,   msgHandle);
 		    if (rc != CWB_OK)
			{
				printf("\ncwbDB_GetDataPointer failed with return ");
				printf("code %d.\n", rc);
				return(rc);
			}
			
			//If this is a CLOB xlate to ascii (the default conversion doesn't apply to lobs)
			if (CWBDB_PCCLOBLOCATOR == columnInf[i].columnClientType)	
			{
				rc =  cwbNL_ConvertCodePages(CWBNL_CP_AS400, CWBNL_CP_CLIENT_ANSI,
											ulLOBLength,	//srclen, 
											ulLOBLength,	//trgtlen,
											lpLOBData,		//srcbuf,
											lpLOBData,		//trgtbuf,
											&nmbrerrs,
											&posoferr,
											&rqdlen, 
											msgHandle); 
				sprintf(szBLOBFn, "C:\\clob%d.txt", i);
			}
			else
				sprintf(szBLOBFn, "C:\\blob%d.bmp", i);		

			/* Save the LOB data off to file. */
			//We'll write out only one row's worth,then overwrite 
			if( (stream = fopen( szBLOBFn, "w+b" )) == NULL )
				printf( "The file %s was not opened\n", szBLOBFn );
			else
			{
				fwrite( lpLOBData,		//pointer to our data
						1,						// 1 byte
						ulLOBLength,	// Number bytes of data
						stream );
				fclose( stream );
				printf("Saved the lob as %s.\n", szBLOBFn);
			}
			cwbDB_DeleteDataHandle(hLOBData, 0);
			break;	//********************* end BLOB/clob locator
			
			case CWBDB_PCBLOB:   
			case CWBDB_PCCLOB:   
	            ulLOBLength = *(long*)dataPtr;	//converted?
				lpLOBData = dataPtr + 4;		//First 4 bytes are lob length
				//If this is a CLOB xlate to ascii (the default conversion doesn't apply to lobs)
				if (CWBDB_PCCLOB == columnInf[i].columnClientType)	
				{
					rc =  cwbNL_ConvertCodePages(CWBNL_CP_AS400, CWBNL_CP_CLIENT_ANSI,
												ulLOBLength,	//srclen, 
												ulLOBLength,	//trgtlen,
												lpLOBData,		//srcbuf,
												lpLOBData,		//trgtbuf,
												&nmbrerrs,
												&posoferr,
												&rqdlen, 
												msgHandle); 
					sprintf(szBLOBFn, "C:\\clob%d.txt", i);
				}
				else
					sprintf(szBLOBFn, "C:\\blob%d.bmp", i);		

				/* Save the LOB data off to file. */
				//We'll write out only one row's worth,then overwrite 
				if( (stream = fopen( szBLOBFn, "w+b" )) == NULL )
					printf( "The file %s was not opened\n", szBLOBFn );
				else
				{
					fwrite( lpLOBData,		//pointer to our data
							1,						// 1 byte
							ulLOBLength,	// Number bytes of data
							stream );
					fclose( stream );
					printf("Saved the lob as %s.\n", szBLOBFn);
				}
				break;		//end lob/clob
			default : 		
				printf("Unknown PC data type of %d", columnInf[i].columnClientType );
         }

         dataPtr = dataPtr + columnInf[i].columnLength;

         printf(" ");

     }
     printf("\n");
   }
   printf("\n");

//  rc = processConversionErrors( indicators, 500, columnCount );

//   printf("There were %d NL conversion errors\n\n", rc );

  return(0);
}


/************************************************************************/
void displayUsage( char *myName )
{

 printf("\nUsage:\n\n\t%s SystemName\n\n", myName );

}
/************************************************************************/

void displayParameterInfo( int           parmNbr,
                           signed short  parmType,
                           unsigned long parmLen  )
{

   printf("\tParameter marker# %d, type: ",parmNbr );
   switch ( parmType )
   {
     case  CWBDB_PCNOCONVERSION : printf("CWBDB_PCNOCONVERSION    Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCSTRING       : printf("CWBDB_PCSTRING          Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCLONG         : printf("CWBDB_PCLONG            Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCSHORT        : printf("CWBDB_PCSHORT           Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCFLOAT        : printf("CWBDB_PCFLOAT           Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCDOUBLE       : printf("CWBDB_PCDOUBLE          Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCPACKED       : printf("CWBDB_PCPACKED          Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCZONED        : printf("CWBDB_PCZONED           Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCINVALIDTYPE  : printf("CWBDB_PCINVALIDTYPE     Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCVARSTRING    : printf("CWBDB_PCVARSTRING       Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCGRAPHIC      : printf("CWBDB_PCGRAPHIC         Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCVARGRAPHIC   : printf("CWBDB_PCVARGRAPHIC      Length: %d \n",
                                  parmLen);
                                  break;
     case  CWBDB_PCBLOBLOCATOR  : printf("CWBDB_PCBLOBLOCATOR     Length: %d \n",
                                  parmLen);
                                  break;								  
     case CWBDB_PCCLOBLOCATOR   : printf("CWBDB_PCCLOBLOCATOR     Length: %d \n",
                                  parmLen);
                                  break;
	 default:
		 printf("Unknown Type %d      Length: %d \n", parmType, parmLen);
   }

}



/************************************************************************/


void displayBaseClientError( unsigned int  clientRC )
{
 printf("Client error::  ");
 switch( clientRC )
 {

  case CWB_USER_CANCELLED_COMMAND          :
       printf("CWB_USER_CANCELLED_COMMAND          \n");
                                             break;
  case CWB_CONFIG_ERROR                    :
       printf("CWB_CONFIG_ERROR\n");
                                             break;
  case CWB_LICENSE_ERROR                   :
       printf("CWB_LICENSE_ERROR\n");
                                             break;
  case CWB_PROD_OR_COMP_NOT_SET            :
       printf("CWB_PROD_OR_COMP_NOT_SET\n");
                                             break;
  case CWB_SECURITY_ERROR                  :
       printf("CWB_SECURITY_ERROR\n");
                                             break;
  case CWB_GLOBAL_CFG_FAILED               :
       printf("CWB_GLOBAL_CFG_FAILED               \n");
                                             break;
  case CWB_PROD_RETRIEVE_FAILED            :
       printf("CWB_PROD_RETRIEVE_FAILED            \n");
                                             break;
  case CWB_COMP_RETRIEVE_FAILED            :
       printf("CWB_COMP_RETRIEVE_FAILED     \n");
                                             break;
  case CWB_COMP_CFG_FAILED                 :
       printf("CWB_COMP_CFG_FAILED                 \n");
                                             break;
  case CWB_COMP_FIX_LEVEL_UPDATE_FAILED    :
       printf(" CWB_COMP_FIX_LEVEL_UPDATE_FAILED   \n");
                                             break;
  case CWB_INVALID_API_HANDLE              :
       printf(" case CWB_INVALID_API_HANDLE             \n");
                                             break;
  case CWB_INVALID_API_PARAMETER           :
       printf("CWB_INVALID_API_PARAMETER     \n");
                                             break;
  case CWB_HOST_NOT_FOUND                  :
       printf("CWB_HOST_NOT_FOUND                   \n");
                                             break;
  case CWB_NOT_COMPATIBLE                  :
       printf("CWB_NOT_COMPATIBLE                   \n");
                                             break;
  case CWB_INVALID_POINTER                 :
       printf("CWB_INVALID_POINTER                  \n");
                                             break;
  case CWB_SERVER_PROGRAM_NOT_FOUND        :
       printf("CWB_SERVER_PROGRAM_NOT_FOUND         \n");
                                             break;
  case CWB_API_ERROR                       :
       printf("CWB_API_ERROR                        \n");
                                             break;
  case CWB_CA_NOT_STARTED                  :
       printf("CWB_CA_NOT_STARTED                   \n");
                                             break;
  case CWB_FILE_IO_ERROR                   :
       printf("CWB_FILE_IO_ERROR                    \n");
                                             break;
  case CWB_COMMUNICATIONS_ERROR            :
       printf("CWB_COMMUNICATIONS_ERROR             \n");
                                             break;
  case CWB_RUNTIME_CONSTRUCTOR_FAILED      :
       printf("CWB_RUNTIME_CONSTRUCTOR_FAILED       \n");
                                             break;
  case CWB_DIAGNOSTIC                      :
       printf("CWB_DIAGNOSTIC                       \n");
                                             break;
  case CWB_COMM_VERSION_ERROR              :
       printf("CWB_COMM_VERSION_ERROR               \n");
                                             break;
  case CWB_NO_VIEWER                       :
       printf("CWB_NO_VIEWER                        \n");
                                             break;
  case CWB_MODULE_NOT_LOADABLE             :
       printf("CWB_MODULE_NOT_LOADABLE              \n");
                                             break;
  case CWB_ALREADY_SETUP                   :
       printf("CWB_ALREADY_SETUP                    \n");
                                             break;
  case CWB_CANNOT_START_PROCESS            :
       printf("CWB_CANNOT_START_PROCESS \n");
                                             break;
//  case CWB_NON_REPRESENTABLE_UNICODE_CHAR  :
       // printf("CWB_NON_REPRESENTABLE_UNICODE_CHAR \n");
                                             break;
  default:
       printf("Unknown error. \n");
     break;
 }
}




/****************************************************************************/
/* A routine to walk through the indicators and check to see if there       */
/* were errors converting between the AS/400 CCSID and the client codepage. */
/*   (These errors can only happen for STRING type data.)                   */
/****************************************************************************/
// unsigned int processConversionErrors( cwbDB_DataHandle indicators,
//                                       unsigned int     rowPosition,
//                                       unsigned int     columnCount )
// {
//  char          *indPtr;
//  signed short  *shortPtr;
//  unsigned long nbrRows = 0, indBuffSize = 0, i = 0,  j = 0, convErrorCnt = 0;
//  char traceMsg[100];
//  cwbSV_TraceDataHandle hTraceData;
//
//
//  cwbDB_GetDataPointer( indicators, &indPtr, NULL );
//
//  shortPtr = (short*)indPtr;
//
//  cwbDB_GetDataLength( indicators, &indBuffSize, NULL  );
//
//  nbrRows = ( indBuffSize / columnCount ) / 2 ;
//
//  cwbSV_CreateTraceDataHandle(CWBREG_ProductName, CWBREG_DBName, &hTraceData);
//
//  for ( j = 0; j < nbrRows; j++ )
//  {
//    for ( i = 1; i <= columnCount ; i++ )
//    {
//      if ( *shortPtr == 0x0045 )
//      {// Log a detailed trace message with the row and column of the
//       // field with the conversion error.
//        if ( convErrorCnt == 0 )
//        {
//          strcpy( traceMsg, "NL CONV ERRORS:\n" );
//          cwbSV_LogTraceData(hTraceData, traceMsg,  strlen(traceMsg));
//        }
//        sprintf( traceMsg, "R %d,C %d           row: %d, column %d\n", j+1+rowPosition, i, j+1+rowPosition, i  );
//        cwbSV_LogTraceData(hTraceData, traceMsg,  strlen(traceMsg));
//        convErrorCnt++;
//       }
//      shortPtr++;
//    }
//  }
//
//  cwbSV_DeleteTraceDataHandle(hTraceData);
//
//
// return convErrorCnt;
//
// }
//
//


/************************************************************************/
/* A routine to cause data to be returned from the AS/400 in ANSI       */
/* instead of OEM.                                                      */
/************************************************************************/
unsigned int causeANSIConversion( cwbDB_FormatHandle dataFormat )
{

    unsigned int           i           = 0;
    unsigned long          columnCount = 0;
    unsigned int           rc          = 0;
    signed short           dataType    = 0;
    unsigned long          dataLength  = 0;
    unsigned long          ansiCCSID   = 0;

    //rc = cwbNL_GetANSICodePage(  &ansiCCSID, 0 );
    if ( rc == CWB_OK )
    {
       rc = cwbDB_GetColumnCount( dataFormat, &columnCount, 0 );
       for ( i = 1; i <= columnCount; i++ )
       {
         rc = cwbDB_GetColumnType( dataFormat, CWBDB_LOCAL, i, &dataType, 0 );
         if ( ( rc == CWB_OK )
              &&  ( ( dataType == CWBDB_PCSTRING )
                  || ( dataType == CWBDB_PCGRAPHIC )
                  || ( dataType == CWBDB_PCVARGRAPHIC )
                  || ( dataType == CWBDB_PCVARSTRING ) ) )
         {
           rc = cwbDB_GetColumnLength( dataFormat, CWBDB_LOCAL, i, &dataLength, 0 );
           rc = cwbDB_SetClientColumnToString( dataFormat, i, dataType, dataLength, ansiCCSID, 0 );
         } /* end if OK */
       } /* end for */
     } /* end NL get codepage OK */
  return rc;
}



