/************************************************************************/
/*                       LICENSE AND DISCLAIMER					                */
/*                                                              		    */
/* This material contains IBM copyrighted sample programming source	    */
/* code for your consideration.  This sample code has not been		      */
/* thoroughly tested under all conditions.  IBM, therefore, cannot	    */
/* guarantee or imply reliability, serviceability, or function.		      */
/* IBM provides no program services for this material. This material	  */
/* is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR	*/
/* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	  */
/* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME		      */
/* JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO	  */
/* THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM	    */
/* BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER  	*/
/* CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,		    */
/* WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS    */
/* OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR	    */
/* OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH	    */
/* DAMAGES.


                          COPYRIGHT                              		    */
/*                              ---------                              	*/
/*               (C) Copyright IBM CORP. 1998, 1999                   	*/
/*               All rights reserved.                                  	*/
/*               US Government Users Restricted Rights -               	*/
/*               Use, duplication or disclosure restricted             	*/
/*               by GSA ADP Schedule Contract with IBM Corp.           	*/
/*               Licensed Material - Property of IBM                   	*/
/*******************************************************************
/*******************************************************************
*
* Module:
*   DBSMGETL.C
*
* Purpose:
*     This is a sample program. It shows the basics of how to use
*     the Client Access/400 Optimized SQL APIs to retrieve information
*     about AS/400 objects (libraries, files, fields, etc).
*
*******************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "CWBDB.H"
#include "CWBSV.H"
#include "DBSMDSPS.H"

#define SYSLEN  14       /* Maximum system name is 14 plus the null char  */
#define LIBLEN  11       /* Maximum library name is 10 plus the null char */
#define PKGLEN  11       /* Maximum SQL package name is 10 plus null char. */
#define FILELEN 11       /* Maximum file name is 10 plus null char. */

// Local functions
unsigned int getLibraryList(  cwbDB_RequestHandle  catRequestHandle,
								              cwbDB_ConnectionHandle connectionHandle,
                              cwbSV_ErrHandle      msgHandle);
unsigned int getFileList(  cwbDB_RequestHandle  catRequestHandle,
								              cwbDB_ConnectionHandle connectionHandle,
                              cwbSV_ErrHandle      msgHandle);
unsigned int getFieldList(  cwbDB_RequestHandle  catRequestHandle,
								              cwbDB_ConnectionHandle connectionHandle,
                              cwbSV_ErrHandle      msgHandle);
void SevereErr(char* api, unsigned int rc);
void GetAndDisplayError(char * msg,
												cwbDB_ConnectionHandle connectionHandle,
												cwbDB_RequestHandle Rhandle,
												unsigned int rc2);

int main(int argc, char *argv[])
{
	cwbDB_ConnectionHandle connectionHandle;   /* Handle to enable connect     */
	cwbDB_RequestHandle    catRequestHandle;   /* Handle to enable rtv reqs    */
	cwbSV_ErrHandle        msgHandle;          /* Handle to PC messages        */
  char                   sysname[SYSLEN],    /* Space for AS/400 system name */
                         *sysptr;            /* Ptr to AS/400 system name    */
  unsigned int           rc;                 /* return codes                 */
  int                    i = 0;
	char                   opt[2];
  
  /* Check on the command line parameters. Expect an AS/400 system name only */
  /* Might get something else or a help request.                             */
    if ( argc < 2 )
    { displayUsage( argv[0] ); return(-1); }
    else
    {
      /* Did user request help? */
      if (*argv[1] == '?' || !strncmp(argv[1], "-h", 2) )
      {
        displayUsage( argv[0] );
        return(-1);
      }
      else { sysptr = argv[1]; } /* Assume system name given. */
    }


    // strtoupper(sysptr);               /* Need the system name in uppercase.   */
    strncpy(sysname, sysptr, SYSLEN); /* Copy it from ARGV to local variable  */

    /* Create PC error message handle. */
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
			SevereErr("cwbSV_CreateErrHandle", rc);

    /********************************************************************/
    /* Create a handle to enable connection to the AS/400. All requests */
    /* and data formats will be tied to a single connection handle.     */
    /*     Think of the connection handle as a collection of associated */
    /*     requests and data formats.                                   */
    /********************************************************************/

    /* Create connection handle to the AS/400 system using the system   */
    /* name passed in to this program.                                  */
    printf("\nCreating connection handle to %s . . .\n", sysptr);
    rc = cwbDB_CreateConnectionHandle( sysptr, &connectionHandle, msgHandle );
    if (rc != CWB_OK)
			SevereErr("cwbDB_CreateConnectionHandle", rc);

    /********************************************************************/
    /* Now that we have a connection handle, we can start a connection  */
    /* with the AS/400 database server.                                 */
    /********************************************************************/

    /* Start communcation between the client and the AS/400.            */
    printf("\nAttempting to start the server . . .");
    rc = cwbDB_StartServer( connectionHandle, msgHandle );
    if (rc  == CWBDB_CANNOT_CONTACT_SERVER )
    {
        printf("\ncwbDB_StartServer failed with return code %d.\n", rc);
        return(rc);
    }
    else
    {
      if ( rc != CWB_OK )
         printf("\ncwbDB_StartServer returned a warning: %d\n", rc );
    }

    printf(" . . . server started.\n");


		// Create a catalog request handle
		// With a catalog request handle we can get information
		// about some of the AS/400 objects.
    rc = cwbDB_CreateCatalogRequestHandle( connectionHandle,
                              &catRequestHandle,  msgHandle  );
    if (rc != CWB_OK)
			SevereErr("cwbDB_CreateConnectionHandle", rc);

		/********************************************************************
		* Now that we have a connection handle and a catalog request handle,
		* we can get some lists of stuff like libraries, files and fields
    ********************************************************************/
		while(opt[0] != '9')
		{
			printf("\nWhat would you like a list of \n(1=library, 2=file, 3=field, 9=quit): ");
			scanf("%s",opt);    
			switch(opt[0])
			{
			case '1':
						{
						getLibraryList(catRequestHandle, connectionHandle, msgHandle);
						break;
						}
			case '2':
						{
						getFileList(catRequestHandle, connectionHandle, msgHandle);
						break;
						}
			case '3':
						{
						getFieldList(catRequestHandle, connectionHandle, msgHandle);
						break;
						}

			default:
					;
			}
		}

		/* End communications between the client and the AS/400. */
		printf("\nEnding the server . . .\n\n");
		if ((rc = cwbDB_StopServer(connectionHandle, msgHandle)) != CWB_OK)
		{
		   printf("\ncwbDB_StopServer failed with return code %d.\n", rc);
       return(rc);
    }
		

return(rc);


} /* end of main */

/************************************************************************/
/***********************************************************************
* This routine gets a library list based on the input.
***********************************************************************/
unsigned int getLibraryList(  cwbDB_RequestHandle  catRequestHandle,
								              cwbDB_ConnectionHandle connectionHandle,
                              cwbSV_ErrHandle      msgHandle)
{
	cwbDB_DataHandle		dataBuffer;
	cwbDB_FormatHandle	dataFormat;
	cwbDB_DataHandle		indicatorsBuffer;
	unsigned int        rc = CWB_OK;        /* return codes                 */
  char                libName[14];
	unsigned int        i;


	/* Create data handles. */
	rc = cwbDB_CreateDataHandle( &dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataFormatHandle( connectionHandle,
		&dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataFormatHandle", rc);

  /***************************************************************************
	* Prompt for library name and then tell the request handle what to look for.
	***************************************************************************/
  printf("Enter a library name with wild characters, or special value like *LIBL: ");
	scanf("%s",libName);
	i = strlen( libName );
	for ( i = 0; i < strlen( libName ) ; i++ )
		libName[i] = (char)toupper(libName[i]);
	printf("\n\n");

	// tell the request handle what libraries you want a list of
  rc = cwbDB_SetLibraryName( catRequestHandle, libName, msgHandle );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_SetLibraryName", rc);

  // Ask the request handle to return data when it comes back
	// from the host.  It seems that you get format anytime you
	// ask for the data.
  rc =  cwbDB_ReturnData( catRequestHandle,
                                       dataBuffer,
                                       indicatorsBuffer,
                                       dataFormat,
                                       msgHandle   );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_ReturnData", rc);

	// ok, send the request handle to the host for the data
	rc = cwbDB_RetrieveLibraryInformation( catRequestHandle, 
				                   CWBDB_GET_LIBRARY_NAME | CWBDB_GET_LIBRARY_DESC,  
													 msgHandle );
	// Had some problems with nulls in the description, so am
	// ignoring conversion errors.
	if ( CWB_OK != rc && rc != CWBDB_NL_CONVERSION_ERROR )
		GetAndDisplayError("cwbDB_RetrieveLibraryInformation",
												connectionHandle,	catRequestHandle, rc);

	// now display the data
	displayData( dataBuffer, dataFormat,  indicatorsBuffer,  msgHandle );

  /* Delete data handles. */
	rc = cwbDB_DeleteDataHandle( dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataFormatHandle( dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);


  return rc;

}     

/************************************************************************/
/***********************************************************************
* This routine gets a list of files in a library based on the input.
***********************************************************************/
unsigned int getFileList(  cwbDB_RequestHandle  catRequestHandle,
								              cwbDB_ConnectionHandle connectionHandle,
                              cwbSV_ErrHandle      msgHandle)
{
	cwbDB_DataHandle		dataBuffer;
	cwbDB_FormatHandle	dataFormat;
	cwbDB_DataHandle		indicatorsBuffer;
	unsigned int        rc = CWB_OK;        /* return codes                 */
	char                libName[14];
  char                fileName[14];
	unsigned int        i;


	/* Create data handles. */
	rc = cwbDB_CreateDataHandle( &dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataFormatHandle( connectionHandle,
		&dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataFormatHandle", rc);

  /***************************************************************************
	* Prompt for library and file name and then tell the request handle what to look for.
	***************************************************************************/
  printf("Enter a library name with wild characters, or special value like *LIBL: ");
	scanf("%s",libName);
	i = strlen( libName );
	for ( i = 0; i < strlen( libName ) ; i++ )
		libName[i] = (char)toupper(libName[i]);
	printf("\n\n");

	// tell the request handle what libraries you want a list of
  rc = cwbDB_SetLibraryName( catRequestHandle, libName, msgHandle );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_SetLibraryName", rc);

	printf("Enter a file name with wild characters or special value like *ALL: ");
	scanf("%s",fileName);
	i = strlen( fileName );
	for ( i = 0; i < strlen( fileName ) ; i++ )
		fileName[i] = (char)toupper(fileName[i]);
	printf("\n\n");

	// tell the request handle what file you want a list of
  rc = cwbDB_SetFileName( catRequestHandle, fileName, msgHandle );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_SetFileName", rc);

  // Ask the request handle to return data when it comes back
	// from the host.  It seems that you get format anytime you
	// ask for the data.
  rc =  cwbDB_ReturnData( catRequestHandle,
                                       dataBuffer,
                                       indicatorsBuffer,
                                       dataFormat,
                                       msgHandle   );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_ReturnData", rc);

	// ok, send the request handle to the host for the data
	rc = cwbDB_RetrieveFileInformation( catRequestHandle, 
				                   CWBDB_GET_FILE_NAME | CWBDB_GET_FILE_DESC,  
													 msgHandle );
	// Had some problems with nulls in the description, so am
	// ignoring conversion errors.
	if ( CWB_OK != rc && rc != CWBDB_NL_CONVERSION_ERROR )
		GetAndDisplayError("cwbDB_RetrieveFieldInformation",
												connectionHandle,	catRequestHandle, rc);

	// now display the data
	displayData( dataBuffer, dataFormat,  indicatorsBuffer,  msgHandle );

/* Delete data handles. */
	rc = cwbDB_DeleteDataHandle( dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataFormatHandle(dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataFormatHandle", rc);

  return rc;
}
     
/************************************************************************/
/***********************************************************************
* This routine gets a list of fields in a file based on the input.
***********************************************************************/
unsigned int getFieldList(  cwbDB_RequestHandle  catRequestHandle,
								              cwbDB_ConnectionHandle connectionHandle,
                              cwbSV_ErrHandle      msgHandle)
{
	cwbDB_DataHandle		dataBuffer;
	cwbDB_FormatHandle	dataFormat;
	cwbDB_DataHandle		indicatorsBuffer;
	unsigned int        rc = CWB_OK;        /* return codes                 */
	char                libName[14];
  char                fileName[14];
	unsigned int        i;


	/* Create data handles. */
	rc = cwbDB_CreateDataHandle( &dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataFormatHandle( connectionHandle,
		&dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataFormatHandle", rc);


/***************************************************************************
	* Prompt for library and file name and then tell the request handle what to look for.
	***************************************************************************/
  printf("Enter a library name: ");
	scanf("%s",libName);
	i = strlen( libName );
	for ( i = 0; i < strlen( libName ) ; i++ )
		libName[i] = (char)toupper(libName[i]);
	printf("\n\n");

	// tell the request handle what libraries you want a list of
  rc = cwbDB_SetLibraryName( catRequestHandle, libName, msgHandle );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_SetLibraryName", rc);

	printf("Enter a file name: ");
	scanf("%s",fileName);
	i = strlen( fileName );
	for ( i = 0; i < strlen( fileName ) ; i++ )
		fileName[i] = (char)toupper(fileName[i]);
	printf("\n\n");

	// tell the request handle what file you want a list of
  rc = cwbDB_SetFileName( catRequestHandle, fileName, msgHandle );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_SetFileName", rc);

  // Ask the request handle to return data when it comes back
	// from the host.  It seems that you get format anytime you
	// ask for the data.
  rc =  cwbDB_ReturnData( catRequestHandle,
                                       dataBuffer,
                                       indicatorsBuffer,
                                       dataFormat,
                                       msgHandle   );
  if ( CWB_OK != rc )
		SevereErr("cwbDB_SetLibraryName", rc);

	// ok, send the request handle to the host for the data
	rc = cwbDB_RetrieveFieldInformation( catRequestHandle, 
				                   CWBDB_GET_FLD_NAME | CWBDB_GET_FLD_DATA_TYPE
													 | CWBDB_GET_FLD_LEN,  
													 msgHandle );
	// Had some problems with nulls in the description, so am
	// ignoring conversion errors.
	if ( CWB_OK != rc && rc != CWBDB_NL_CONVERSION_ERROR )
		GetAndDisplayError("cwbDB_RetrieveFieldInformation",
												connectionHandle,	catRequestHandle, rc);

	// now display the data
	displayData( dataBuffer, dataFormat,  indicatorsBuffer,  msgHandle );

/* Delete data handles. */
	rc = cwbDB_DeleteDataHandle( dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataFormatHandle(dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataFormatHandle", rc);


  return rc;

}     

/********************************************************************/
/* When we do not ask for the error information to be returned		  */
/* and there is an error which we would like information about	   	*/
/* then we must ask for the error information and do a get data   	*/
/* to cause a flow the the host to get the error information.	    	*/
/********************************************************************/
void GetAndDisplayError(char * msg,
												cwbDB_ConnectionHandle connectionHandle,
												cwbDB_RequestHandle Rhandle,
												unsigned int rc2)
{
	cwbSV_ErrHandle 		 msgHandle;
	cwbDB_DataHandle		 hostMsgID; 		 /* Host message identifier. */
	cwbDB_DataHandle		 firstLevelMessage;  /* First level message text. */
	cwbDB_DataHandle		 secondLevelMessage; /* Second level message text. */
	unsigned int			 rc = 0;
	unsigned short			 hostErrorClass = 0;
	signed long 			 hostErrorCode = 0;
	
	/* Create error handles. */
	rc = cwbSV_CreateErrHandle( &msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateErrHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	// Ask for error information and get data
	rc =  cwbDB_ReturnHostErrorInfo( Rhandle,
		&hostErrorClass,
		&hostErrorCode,
		hostMsgID,
		firstLevelMessage,
		secondLevelMessage,
		msgHandle				 );
	if (rc != CWB_OK)
		SevereErr("cwbDB_ReturnHostErrorInfo", rc);
	
	// this will cause a flow to the host which will retreive the
	// error information fot the request handle.
	rc = cwbDB_GetData( Rhandle, msgHandle );
	if (rc != CWB_OK)
		   SevereErr("cwbDB_ReturnHostErrorInfo", rc);
	else
		displayErrors( msg, rc2, hostErrorClass, hostErrorCode,
			hostMsgID, firstLevelMessage, secondLevelMessage );
	exit(rc2);
}


/********************************************************************/
/* An API got an error that will keep the application from running	*/
/* so print the API name and return code and stop execution.		*/
/*																	*/
/********************************************************************/
void SevereErr(char* api, unsigned int rc)
{
	printf("\n%s failed with return code %d.\n", api, rc);
	exit(rc);
}

