/************************************************************************/
/*                       LICENSE AND DISCLAIMER					                */
/*                                                              		    */
/* This material contains IBM copyrighted sample programming source	    */
/* code for your consideration.  This sample code has not been		      */
/* thoroughly tested under all conditions.  IBM, therefore, cannot	    */
/* guarantee or imply reliability, serviceability, or function.		      */
/* IBM provides no program services for this material. This material	  */
/* is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR	*/
/* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	  */
/* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME		      */
/* JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO	  */
/* THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM	    */
/* BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER  	*/
/* CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,		    */
/* WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS    */
/* OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR	    */
/* OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH	    */
/* DAMAGES.


                          COPYRIGHT                              		    */
/*                              ---------                              	*/
/*               (C) Copyright IBM CORP. 1998, 1999                   	*/
/*               All rights reserved.                                  	*/
/*               US Government Users Restricted Rights -               	*/
/*               Use, duplication or disclosure restricted             	*/
/*               by GSA ADP Schedule Contract with IBM Corp.           	*/
/*               Licensed Material - Property of IBM                   	*/
/*******************************************************************
/*******************************************************************
*
* Module:
*   DBSMINS.C
*
* Purpose:
*
*    Shows how to create a file, override members and use block insert
*    to put records into the different members.
*
*    Note that library DBAPITEST must exist for the program to run.
*    Addtionaly file BLKINSRT , which is created by this program 
*    must not exist in that file.
*******************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "CWBDB.H"       /* DB API include file     */
#include "CWBSV.H"       /* Sevice API include file */
#include "dbsmdsps.h"

#define SYSLEN  14       /* Maximum system name is 14 plus the null char    */
#define LIBLEN  11       /* Maximum library name is 10 plus the null char   */
#define PKGLEN  11       /* Maximum SQL package name is 10 plus null char.  */
#define FILELEN 11       /* Maximum file name is 10 plus null char.         */

/* Displays the required parameters to pass to this program. */
void myUsage(void);

/* A running monologue of the program if the program is compiled with */
/* the variable VERBOSE set.                                          */
void verbose( char * myString );


/* Cycles through the parameter markers, calling displayParameterInfo */
/* for each parameter marker and changing packed and zoned numeric    */
/* parameters to PC long variables.                                   */
unsigned int workWithParameterMarkers( cwbDB_FormatHandle pmDataFormat );


/* This routine is used to acquire data for upload. */
void readData( char *dataBlock,      unsigned long rowSize,
               char *indicatorBlock, unsigned long nbrParms );

/* This routine creates a collection (library) on the AS/400. */
unsigned int createCollection( char *libName, cwbDB_RequestHandle sqlRequest );

/* This routine deletes a collection (library) or a table (file) on the AS/400. */
/* Dangerous stuff!                                                             */
/*   objName      = 1-10 character name of collection or table.                 */
/*   sqlRequest   = previously created SQL request handle.                      */
/*   objType      = 'C' for collection, 'T' for table. 'T' is default.          */
unsigned int dropIt( char *objName, cwbDB_RequestHandle sqlRequest, char objType );

/* This routine creates a member in a file. Expects some data to be previously  */
/* stored on the AS/400 in the data buffer associated with the request handle.  */
unsigned int createMember( char *mbrName,  /* Name of member to be created      */
                           char *mbrText,  /* Text desciption of member created */
                           cwbDB_RequestHandle ndbRequestHandle ); /* handle    */

int main(int argc, char *argv[])
{

    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.  */
    cwbDB_RequestHandle      ndbRequestHandle,   /* Object manipulation handle.  */
                             sqlRequestHandle;   /* SQL operation handle.        */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.     */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.    */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.   */
    cwbDB_ConnectionHandle   connectionHandle;   /* Handle to enable conneciton. */
    cwbDB_FormatHandle       pmDataFormat;       /* Data format handle for       */
                                                 /* parameter marker data.       */
    char                     sysname[SYSLEN],    /* system name.                 */
                             *sysptr,            /* system name ptr.             */
                             tableName[FILELEN], /* file name                    */
                             collectionName[LIBLEN], /* library name             */
                             memberName[FILELEN],/* member name                  */
                             text[29];           /* lib/file/mbr text            */
    unsigned int             rc;                 /* return codes                 */
    int                      i = 0, k = 0;       /* miscellaneous integers       */
    unsigned long            j = 0,
                             parmLength = 0,     /* parameter marker length          */
                             nbrParms = 0;       /* number of parameter markers      */
    unsigned short           parmPrecision = 0,  /* parameter marker precision       */
                             parmScale = 0;      /* parameter marker scale           */
    signed short             columnType = 0;     /* data type of a column            */
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error   */
    signed long              hostErrorCode = 0;  /* AS/400 operation return code     */
    char                     *dataBlock;         /* pointer to data buffer           */
    char                     *indicatorBlock;    /* pointer to null indicator buffer */


  /* Check on the command line parameters. Expect an AS/400 system name only */
  /* Might get something else or a help request.                             */
    if (argc > 1)
    {
        /* Did user request help? */
        if (*argv[1] == '?' || !strncmp(argv[1], "-h", 2))
        {
            myUsage();
            return(-1);
        }
        else { sysptr = argv[1]; } /* Assume system name given. */
    }
    else { myUsage(); return -1; }

    /* Copy system name from ARGV to local variable  */
    strncpy(sysname, sysptr, SYSLEN);

    /* Create PC error message handle. This handle is used by the API layer      */
    /* to log serviceability messages.                                           */
    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create a handle to enable connection to the AS/400. SQL requests */
    /* and data formats will be tied to a single connection handle.     */
    /*     Think of the connection handle as a collection of associated */
    /*     requests and data formats.                                   */
    /********************************************************************/

    /* Create connection handle to the AS/400 system using the system   */
    /* name passed in to this program.                                  */
    verbose(">>> Create connection handle to the AS/400 system. \n");
    printf("\nCreating connection handle to %s . . .\n", sysptr);
    rc = cwbDB_CreateConnectionHandle( sysptr, &connectionHandle, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateConnectionHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Now that we have a connection handle, we can start a connection  */
    /* with the AS/400 database server.                                 */
    /********************************************************************/

    /* Start communications between the client and the AS/400. */
    verbose(">>> Start communcation between the client and the AS/400. \n");
    printf("\nAttempting to start the server . . .");
    rc = cwbDB_StartServer( connectionHandle, msgHandle );
    if (rc  != CWB_OK)
    {
        printf("\ncwbDB_StartServer failed with return code %d.\n", rc);
        return(rc);
    }

    printf(" started\n\n");

    /********************************************************************/
    /* Now that we have a connection handle, we can create the request  */
    /* handles and data format handles which will be associated with it.*/
    /********************************************************************/

    /* Create NDB request handle for subsequest API calls that request */
    /* operations to be performed with AS/400 file objects.            */
    verbose(">>> Create NDB request handle for subsequest API calls that request \n");
    rc = cwbDB_CreateNDBRequestHandle( connectionHandle,
                                       &ndbRequestHandle,
                                       msgHandle);
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateNDBRequestHandle failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Create SQL request handle for the subsequent APIs that request SQL */
    /* operations to be performed.                                        */
    verbose(">>> Create SQL request handle for the subsequent APIs that request SQL \n");
    rc = cwbDB_CreateSQLRequestHandle( connectionHandle,
                                       &sqlRequestHandle,
                                       msgHandle         );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateSQLRequestHandle failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }


/*******************************************************************/
/* Set up thus far...                                              */
/*******************************************************************/

    /*******************************************************************/
    /* First step here is to create a file to insert data into later.  */
    /* In order to do this, we need to set the library name to create  */
    /* the file in, and the name of the file to create.                */
    /*******************************************************************/

    /* Set the name of the DB file to be created. */
    strcpy( tableName, "BLKINSRT" );

    /* Set the file name for the DB file create */
    verbose(">>> Set the file name for the DB file create \n");
    rc = cwbDB_SetFileName( ndbRequestHandle,
                            tableName,
                            msgHandle         );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetFileName failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Set the library name to contain the DB file. */
    strcpy( collectionName, "DBAPITEST" );

    /* Set the name of the library to contain the file. */
    verbose(">>> Set the library name for the DB file create \n");
    rc = cwbDB_SetLibraryName( ndbRequestHandle,
                               collectionName,
                               msgHandle         );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetLibraryName failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /*************************************************************************/
    /* Next we need to set the length of the record in the file, a brief     */
    /* description of the file, and a brief description of the first member. */
    /*************************************************************************/

    /* Set the record length for the DB file create */
    verbose(">>> Set the record lengthfor the DB file create \n");
    rc = cwbDB_SetRecordLength( ndbRequestHandle,
                                80,
                                msgHandle         );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetRecordLength failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Set some arbitrary text for a description */
    sprintf( text, "%s is a test file.", tableName );

    /* Set the description for the DB file created */
    verbose(">>> Set the description for the DB file created \n");
    rc = cwbDB_SetFileText( ndbRequestHandle, text, msgHandle         );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetFileText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Set some arbitrary text for the member description */
    strcpy( text, "First member");

    /* Set the description for the DB file member created */
    verbose(">>> Set the description for the DB file member created \n");
    rc = cwbDB_SetMemberText( ndbRequestHandle, text, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetMemberText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Part of the 'philosophy' of the optimized SQL APIs is that       */
    /* nothing is returned from the AS/400 unless it is requested by    */
    /* the client. This reduces communications traffic in order to      */
    /* improve performance. In order to request data from the AS/400,   */
    /* call one of the cwbDB_Return...... APIs to request the data      */
    /* and then perform a function (verb) such as cwbDB_Prepare, or     */
    /* call cwbdb_GetData.                                              */
    /********************************************************************/

    /* Ask for the host info to be returned. This is the only way to    */
    /* find out if an error occurred on the following operation.        */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );


    /********************************************************************/
    /* At times it is useful to store some of the operation parameters  */
    /* in a block on the AS/400 so that we don't have to send them up   */
    /* again later. The block they are stored in is called RPB: Request */
    /* Parameter Block.                                                 */
    /********************************************************************/

    /* Store the parameters in the NDB RPB */
    verbose(">>> Store the parameters in the NDB RPB \n");
    rc = cwbDB_StoreRequestParameters( ndbRequestHandle, msgHandle  );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_StoreRequestParameters",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }


    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /*********************************************************************/
    /* Use the parameters stored in the RPB to perform an operation.     */
    /*********************************************************************/
    /* Create the source physical file. The first member will be created */
    /* at the same time.                                                 */
    /*********************************************************************/
    verbose(">>> Create the source physical file \n");
    rc = cwbDB_CreateSourcePhysicalFile( ndbRequestHandle, msgHandle );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_CreateSourcePhysicalFile",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

    /***************************************************************************/
    /* Now we are going to create a second member in the source physical file. */
    /***************************************************************************/

    strcpy( memberName, "MBR2" );
    strcpy( text, "Second member" );
    rc = createMember( memberName, text, ndbRequestHandle );
    if ( rc != CWB_OK )
      return(rc);

    /***************************************************************************/
    /* Now we are going to create a third member in the source physical file.  */
    /***************************************************************************/
    strcpy( memberName, "MBR3" );
    strcpy( text, "Third member" );
    rc = createMember( memberName, text, ndbRequestHandle );
    if ( rc != CWB_OK )
    {  return(rc); }


    /***************************************************************************/
    /* Now we are going to create a fourth member in the source physical file. */
    /***************************************************************************/
    strcpy( memberName, "MBR4" );
    strcpy( text, "Fourth member" );
    rc = createMember( memberName, text, ndbRequestHandle );
    if ( rc != CWB_OK )
    { return(rc); }

    /***********************************************************************/
    /* Ok. The source physical file is created and we've even added a few  */
    /* members. We want to upload some data to one of the members. Because */
    /* it's easy to upload to the first, or default, member we'll be       */
    /* uploading to another member to show you how to use other APIs.      */
    /***********************************************************************/
    /* In order to get at members other than the first member using SQL    */
    /* (SQL only knows about the first member in a file) we have to create */
    /* an override to use when accessing the other members.                */
    /***********************************************************************/

    /***********************************************************************/
    /* We need a name to refer to the override later on when we want to    */
    /* use it. Here we're going to use the name TESTSTF.                   */
    /***********************************************************************/
    verbose(">>> Set the file name for the DB file to override \n");
    rc = cwbDB_SetFileName( ndbRequestHandle,  "TESTSTF",  msgHandle  );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetFileName failed with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /***********************************************************************/
    /* Now name the library, file, and member we want to override with the */
    /* name TESTSTF later on. In this case we'll be overriding the second  */
    /* member in the file we just created.                                 */
    /* Put another way, TESTSTF == collectionName/tableName(memberName)    */
    /***********************************************************************/
    strcpy( memberName, "MBR3" );    /* Going to override the third member */
    verbose(">>> Set override info \n");
    rc = cwbDB_SetOverrideInformation( ndbRequestHandle,
                                       collectionName,
                                       tableName,
                                       memberName,
                                       msgHandle         );
    if ( rc != 0 )
    {
        displayErrors( "cwbDB_SetOverrideInformation",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
       return(rc);
    }

    /* Store the parameters in the NDB RPB */
    verbose(">>> Store the parameters in the RPB \n");
    rc = cwbDB_StoreRequestParameters( ndbRequestHandle,  msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_StoreRequestParameters failed with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /***************************************************************************/
    /* Perform the actual override operation.                                  */
    /***************************************************************************/
    verbose(">>> Override the lib/file/mbr \n");
    rc = cwbDB_OverrideFile( ndbRequestHandle,  msgHandle );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_OverrideFile",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

    /****************************************************************************/
    /* Now we can use the name TESTSTF to perform SQL operations on             */
    /* collection/table(member)                                                 */
    /****************************************************************************/
    /* BEGIN INSERT OPERATION. THIS FIRST PART ONLY NEEDS TO BE DONE ONCE PER   */
    /* UPLOAD. it's set-up stuff for the later SQL inserts.                    */
    /****************************************************************************/

    /* Set the statement text in the SQL request.                               */
    verbose(">>> Set statement text \n");
    rc = cwbDB_SetStatementName( sqlRequestHandle, "STMTONE", msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementName failed with return code %d.\n", rc);
        return(rc);
    }

    /****************************************************************************/
    /* All SQL operations require the cursor name and the statement name.       */
    /****************************************************************************/
    /*   statement name (arbitrary up to 18 character name)                     */
    /*                  The database server uses the statement name             */
    /*                  during the PREPARE function in one of its               */
    /*                  optimization steps.                                     */
    /*   cursor name    (arbitrary up to 18 character name)                     */
    /*                  Think of the cursor as a pointer in the                 */
    /*                  database table which points at a row of data.           */
    /****************************************************************************/
    /* Set the name of the cursor */
    verbose(">>> Set cursor name \n");
    rc = cwbDB_SetCursorName( sqlRequestHandle, "STFCUR1", msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetCursorName failed with return code %d.\n", rc);
        return(rc);
    }

   /*****************************************************************************/
   /* We need to use an INSERT statement to upload data. There are three ways   */
   /* we can use the INSERT statement:                                          */
   /*   1) imbed the data within the SQL INSERT statement and upload each row   */
   /*      of data separately:                                                  */
   /*           INSERT INTO TESTSTF VALUES( 1, 2, "Single row of data here." )  */
   /*   2) use parameter markers for the data and upload each row of data       */
   /*      separately:                                                          */
   /*           INSERT INTO TESTSTF VALUES( ?, ?, ? )                           */
   /*   3) use parameter markers for the data and upload multiple rows of data: */
   /*           INSERT INTO TESTSTF ? ROWS VALUES( ?, ?, ? )                    */
   /* In this sample, we'll use the latter method to show how it's done.        */
   /*****************************************************************************/

   /*****************************************************************************/
   /* Here's the insert statement. Notice the extra ? ROWS clause.              */
   /* This is what tells SQL that we're inserting multiple rows of data.        */
   /* Also, there are the question marks in the VALUES clause because our       */
   /* source physical file has three fields.                                    */
   /*****************************************************************************/
    verbose(">>> Set statement text \n");
    rc = cwbDB_SetStatementText( sqlRequestHandle, "INSERT INTO TESTSTF ? ROWS VALUES( ?, ?, ?  )", msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );


    /* Store the parameters in the SQL RPB */
    verbose(">>> Store the parameters in the SQL RPB \n");
    rc = cwbDB_StoreRequestParameters( sqlRequestHandle,
                                       msgHandle         );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_StoreRequestParameters",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /*****************************************************************************/
    /* PREPARE the INSERT statement. This checks the syntax and creates the      */
    /* necessary access links on the AS/400.                                     */
    /*****************************************************************************/
    verbose(">>> Prepare the INSERT statement \n");
    rc = cwbDB_Prepare( sqlRequestHandle,  msgHandle );
    if ( rc != CWB_OK )
    {
        displayErrors( "cwbDB_Prepare",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

  /********************************************************************/
  /* The parameter marker data format provides a way of describing    */
  /* the parameter marker variables. The parameter marker data format */
  /* has information about each column such as the column data type,  */
  /* the column data length, the column name...                       */
  /* The data format also has information about a row of data such    */
  /* as the size of a row of data.                                    */
  /********************************************************************/
  /* Here we are creating the parameter marker data format handle.    */
  /********************************************************************/
  rc = cwbDB_CreateParameterMarkerFormatHandle( connectionHandle,
                                                &pmDataFormat,
                                                msgHandle        );
  if (rc != CWB_OK)
  {
      printf("\ncwbDB_CreateParameterMarkerFormatHandle failed with return code %d.\n", rc);
      return(rc);
  }

  /********************************************************************/
  /* Now we'll ask the AS/400 to return the actual data format        */
  /* based on the results of a describe operation on the INSERT       */
  /* statement parameter markers.                                     */
  /********************************************************************/
  verbose(">>> Ask for the PM data format to be returned. \n");
  rc =  cwbDB_ReturnParameterMarkerFormat( sqlRequestHandle,
                                           pmDataFormat,
                                           msgHandle          );
  if (rc != CWB_OK)
  {
      printf("\ncwbDB_ReturnParameterMarkerFormat failed with return code %d.\n", rc);
      return(rc);
  }

  /*********************************************************************/
  /* Now we'll ask the AS/400 to perform the parameter marker describe */
  /* operation on the INSERT statement.                                */
  /*********************************************************************/
  verbose(">>> DescribeParameterMarkers the INSERT statement \n");
  rc = cwbDB_DescribeParameterMarkers( sqlRequestHandle,
                                    msgHandle         );
  if ( rc != CWB_OK )
  {
      displayErrors( "cwbDB_DescribeParameterMarkers",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
      return(rc);
  }


  /*********************************************************************/
  /* Find out how many parameter markers there were in the INSERT      */
  /* statement. For this exercise, it will return 3. The ? before the  */
  /* ROWS clause is not included in this count.                        */
  /*********************************************************************/
  rc = cwbDB_GetParameterCount( pmDataFormat,  (unsigned long*)&nbrParms, msgHandle     );
  if ( rc != CWB_OK )
  { return(rc); }

  /***************************************************************/
  /* This next call is to display info about each parameter      */
  /* and change packed or zoned numerics to a more PC            */
  /* friendly format.                                            */
  /* It's not necessary to the operation of the upload, just     */
  /* an interesting aside.                                       */
  /***************************************************************/
  rc = workWithParameterMarkers( pmDataFormat );
  /***************************************************************/
  /* We now return to the regularly scheduled programming. . .   */
  /***************************************************************/


  rc = cwbDB_SetClientParameterToString( pmDataFormat,
                                         3,
                                         CWBDB_PCSTRING,
                                         40,
                                         437,
                                         0 );

/* THIS NEXT CHUNK YOU'LL PROBABLY WANT TO PUT INTO A LOOP  UNLESS     */
/* YOU ARE LUCKY ENOUGH TO HAVE ALL OF THE DATA IN ONE BUFFER ALREADY. */

    /* Get the length of one row of input */
    rc = cwbDB_GetSizeOfInputParameters( pmDataFormat,  CWBDB_LOCAL, &parmLength, msgHandle     );
    if ( rc != CWB_OK )
    {
      printf( "\ncwbDB_GetSizeOfInputParameters failed with return code: %d \n", rc );
      return(rc);
    }

    /*******************************************************************/
    /* I want to insert 3 rows of data, so multiply size of input by 3 */
    /* and allocate a buffer.                                          */
    /*******************************************************************/
    dataBlock = (char *) malloc( parmLength * 3 );

    /* Always a good idea to clear the memory */
    memset( dataBlock,0,parmLength*3);

    /**************************************************************************/
    /* Each parameter may possibly be a null capable field, and thus need     */
    /* a nullindicator. Each null indicator is two bytes. I'm assuming 3      */
    /* rows of input, so nbrParms * size of null indicator * nbrRows is the   */
    /* size of the indicator buffer.                                          */
    /**************************************************************************/
    indicatorBlock = (char*) malloc( nbrParms * 2 * 3 );

    /* Always a good idea to clear the memory */
    memset( indicatorBlock,0,nbrParms*2*3);

    /***************************************************************************/
    /* The next routine could be used to read data from a file.                */
    /* For this sample, all it does is generate 3 rows of data to be inserted. */
    /***************************************************************************/
    readData( dataBlock, parmLength, indicatorBlock, nbrParms );

    /* Set      the block of data */
    verbose(">>> Set parameter marker block\n");
    rc = cwbDB_SetParameterMarkerBlock( sqlRequestHandle,
                                        3, /* remember, I want 3 rows inserted */
                                        pmDataFormat,
                                        (void*)dataBlock, // NULL,
                                        (signed short *)indicatorBlock,
                                        msgHandle );

    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetParameterMarkerBlock failed. rc: %d ",rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_ReturnHostErrorInfo failed. rc: %d ",rc);
        return(rc);
    }

    /**********************************************************************/
    /* NOTE: Cannot store parameter marker data in the cached parameters! */
    /**********************************************************************/

    /* Execute the INSERT statement to put the data into the file. */
    verbose(">>> Execute the INSERT statement \n");
    rc = cwbDB_Execute( sqlRequestHandle,
                        msgHandle         );
    if (rc != CWB_OK)
    {
      displayErrors( "cwbDB_Execute",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
      return(rc);
    }


  /* IF YOU MAKE THIS INTO A LOOP, END THE LOOP HERE. */

    /* End communications between the client and the AS/400. */
    verbose(">>> End communications between the client and the AS/400. \n");
    printf("\nEnding the server . . .");
    if ((rc = cwbDB_StopServer(connectionHandle, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_StopServer failed with return code %d.\n", rc);
        return(rc);
    }


return(rc);

} /* end of main */

/************************************************************************/
void myUsage(void)
{


        printf("\n\n\n\nThis program uses the Client Access/400 DB APIs to INSERT a block\n");
        printf("of data into a database file on the AS/400.\n\n\n\n" );
        printf("\nThis program requires a system name to be passed as a parameter. \n\n\n" );


}
/************************************************************************/
void verbose( char * myString )
{
#if defined(VERBOSE)
  printf("%s",myString);
#endif
}

/************************************************************************/
/* A routine to "read" data in from a file. For simplicity, it just     */
/* creates a little data on its own.                                    */
/************************************************************************/
void readData( char *dataBlock,      unsigned long rowSize,
               char *indicatorBlock, unsigned long nbrParms )

{

    /*******************************************************************************/
    /* This next chunk is for building my dummy data; you don't need this   <<<<<< */
    /*                                                                      <<<<<< */

    /* You would probably want to replace it with code to read              <<<<<< */
    /* from a data file.                                                    <<<<<< */
    /*                                                                      <<<<<< */
    unsigned long            j = 0;                                     /*  <<<<<< */
    long                     *pLong, i =0;                              /*  <<<<<< */
    char                     *dataString;                               /*  <<<<<< */
    unsigned long            parmLength = rowSize;                      /*  <<<<<< */
    signed short             *indicatorPtr;                             /*  <<<<<< */
    /*                                                                      <<<<<< */
    pLong = (long*)dataBlock;                                           /*  <<<<<< */
    *pLong = 1;                                                         /*  <<<<<< */
    pLong = (long*)(dataBlock+sizeof(long));                            /*  <<<<<< */
    *pLong = 1;                                                         /*  <<<<<< */
    dataString = dataBlock+sizeof(long)+sizeof(long);                   /*  <<<<<< */
    strcpy(dataString,"DSPMSG QSYSOPR");                                /*  <<<<<< */
    i = strlen(dataString);                                             /*  <<<<<< */
    memset((dataString+i),' ',68-i);                                    /*  <<<<<< */
                                                                        /*  <<<<<< */
    pLong = (long*)(dataBlock+(1*parmLength));                          /*  <<<<<< */
    *pLong = 2;                                                         /*  <<<<<< */
    pLong = (long*)(dataBlock+sizeof(long)+(1*parmLength));             /*  <<<<<< */
    *pLong = 2;                                                         /*  <<<<<< */
    dataString = dataBlock+(1*parmLength)+sizeof(long)+sizeof(long);    /*  <<<<<< */
    strcpy(dataString,"WRKCTLD *ALL");                                  /*  <<<<<< */
    i = strlen(dataString);                                             /*  <<<<<< */
    memset((dataString+i),' ',68-i);                                    /*  <<<<<< */
                                                                        /*  <<<<<< */
    pLong = (long*)(dataBlock+(2*parmLength));                          /*  <<<<<< */
    *pLong = 3;                                                         /*  <<<<<< */
    pLong = (long*)(dataBlock+sizeof(long)+(2*parmLength));             /*  <<<<<< */
    *pLong = 3;                                                         /*  <<<<<< */
    dataString = dataBlock+(2*parmLength)+sizeof(long)+sizeof(long);    /*  <<<<<< */
    strcpy(dataString,"WRKSPLF");                                       /*  <<<<<< */
    i = strlen(dataString);                                             /*  <<<<<< */
    memset((dataString+i),' ',68-i);                                    /*  <<<<<< */
    /*  End of building dummy data                                          <<<<<< */
    /*                                                                      <<<<<< */
    /* Now it's necessary to build a set of null indicators  for the data.  <<<<<< */
    /* Each indicator is two bytes, and there must be one for every field   <<<<<< */
    /* in every row of data to be inserted (#rows*#columns*2)               <<<<<< */
    /* 0 = field is not null, -1 == field is null                           <<<<<< */
    /*                                                                      <<<<<< */
    indicatorPtr = (unsigned short*)indicatorBlock;                     /*  <<<<<< */
    for ( i = 1; i <=3; i++)                                            /*  <<<<<< */
    {                                                                   /*  <<<<<< */
      for ( j = 0; j < nbrParms; j++ )                                  /*  <<<<<< */
      {                                                                 /*  <<<<<< */
        *indicatorPtr = 0;                                              /*  <<<<<< */
        indicatorPtr += 1;                                              /*  <<<<<< */
      }                                                                 /*  <<<<<< */
    }                                                                   /*  <<<<<< */
    /*******************************************************************************/
} /* end readData */



unsigned int createCollection( char *libName, cwbDB_RequestHandle sqlRequest )
{
    unsigned int             rc = 0;
    char                     stmt[200];          /* SQL statement buffer          */
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error*/
    signed long              hostErrorCode = 0;  /* AS/400 operation return code  */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.   */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.      */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.     */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.    */

    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    sprintf( stmt, "CREATE COLLECTION %S", libName );

    rc = cwbDB_SetStatementText( sqlRequest, stmt, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Execute the CREATE COLLECTION statement to put the data into the file. */
    verbose(">>> Execute the CREATE statement \n");
    rc = cwbDB_Execute( sqlRequest, msgHandle );
    if (rc != CWB_OK)
    {
      displayErrors( "cwbDB_Execute",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
    }


  return(rc);

}  /* end createCollection  */

/*******************************************************************************/
/* This routine creates a member in a file. Expects some data to be previously */
/* stored on the AS/400 in the data buffer associated with the request handle. */
/*******************************************************************************/
unsigned int createMember( char *mbrName, char *mbrText, cwbDB_RequestHandle ndbRequestHandle )
{
    unsigned int             rc = 0;
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error*/
    signed long              hostErrorCode = 0;  /* AS/400 operation return code  */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.   */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.      */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.     */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.    */

    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /***************************************************************************/
    /* Now we are going to create a second member in the source physical file. */
    /***************************************************************************/
    /* Set the _description_ for the next DB file member to create             */
    verbose(">>> Set the description for the next DB file member created \n");
    rc = cwbDB_SetMemberText( ndbRequestHandle, mbrText, msgHandle  );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetMemberText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Set the _name_ for the next DB file member created */
    verbose(">>> Set the name for the next DB file member created \n");
    rc = cwbDB_SetMemberName( ndbRequestHandle, mbrName,  msgHandle  );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetMemberName failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /***********************************************************************/
    /* Use the parameters stored in the RPB along with the new parameters  */
    /* just set to perform the next operation.                             */
    /***********************************************************************/
    /* Create the next member                                              */
    /***********************************************************************/
    verbose(">>> Create the next member \n");
    rc = cwbDB_AddMember( ndbRequestHandle,  msgHandle  );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_AddMember",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

  return(rc);
}   /* end createMember */


/****************************************************************/
/* This routine steps through the parameter marker data format, */
/* displaying information about each parameter marker and       */
/* changing zoned and packed numerics to PC longs for           */
/* convenience.                                                 */
/****************************************************************/
unsigned int workWithParameterMarkers( cwbDB_FormatHandle pmDataFormat )
{

    unsigned int             rc;                 /* return codes                     */
    int                      k = 0;              /* miscellaneous integers           */
    unsigned long            i= 0, j = 0,
                             parmLength = 0,     /* parameter marker length          */
                             nbrParms = 0;       /* number of parameter markers      */
    unsigned short           parmPrecision = 0,  /* parameter marker precision       */
                             parmScale = 0;      /* parameter marker scale           */
    signed short             columnType = 0;     /* data type of a column            */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.      */


    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /*********************************************************************/
    /* Find out how many parameter markers there were in the INSERT      */
    /* statement. For this exercise, it will return 3. The ? before the  */
    /* ROWS clause is not included in this count.                        */
    /*********************************************************************/
    rc = cwbDB_GetParameterCount( pmDataFormat,  (unsigned long*)&nbrParms, msgHandle     );
    if ( rc != CWB_OK )
    { return(rc); }


    j = nbrParms;

    /***********************************************************/
    /* This next loop is to display info about each parameter  */
    /* and change packed or zoned numerics to a more PC        */
    /* friendly format.                                        */
    /* It's not necessary to the operation of the upload, just */
    /* an interesting aside.                                   */
    /***********************************************************/

    for (i = 1; i <= j ;i++ )
    {

      /* Get the PC data type of the parameter  */
      rc = cwbDB_GetParameterType( pmDataFormat, CWBDB_LOCAL, i, &columnType, msgHandle     );
      if ( rc != CWB_OK )
      {
        printf( "\ncwbDB_GetParameterType failed for parameter marker %d with return code: %d \n", i, rc );
        return(rc);
      }

      /* Get the length of the parameter */
      rc = cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, i, &parmLength,  msgHandle     );
      if ( rc != CWB_OK )
      {
        printf( "\ncwbDB_GetParameterLength failed for parameter marker %d with return code: %d \n", i, rc );
        return(rc);
      }


     /* Massage the packed and zoned numerics. */
     if (  ( columnType == CWBDB_PCPACKED )
        || ( columnType == CWBDB_PCZONED  ) )
        { /* Is it a packed or zoned numeric?         */
          /* If so, what are the precision and scale? */
          /* We'll need them to convert parameter to  */
          /* PC LONG type for our convenience.        */
          rc = cwbDB_GetParameterPrecision( pmDataFormat, CWBDB_LOCAL, i, &parmPrecision, msgHandle );
          if ( rc != CWB_OK )
          {
            printf( "\ncwbDB_GetParameterPrecision failed for parameter marker %d with return code: %d \n", i, rc );
            return(rc);
          }

          rc = cwbDB_GetParameterScale( pmDataFormat, CWBDB_LOCAL, i, &parmScale, msgHandle );
          if ( rc != CWB_OK )
          {
            printf( "\ncwbDB_GetParameterScale failed for parameter marker %d with return code: %d \n", i, rc );
            return(rc);
          }

          /* It's easier to use a LONG on the PC, so change the type. */
          rc = cwbDB_SetClientParameterToNumeric( pmDataFormat,
                                                  i,
                                                  CWBDB_PCLONG,
                                                  parmLength,
                                                  parmPrecision,
                                                  parmScale,
                                                  msgHandle );
          if ( rc != CWB_OK )
          {
            printf( "\ncwbDB_SetClientParameterToNumeric failed for parameter marker %d with return code: %d \n", i, rc );
            return(rc);
          }
          columnType = CWBDB_PCLONG;

        }

      displayParameterInfo( i, columnType, parmLength );

    } /* endfor */

  return(rc);

} /* end workWithParameterMarkers */


unsigned int dropIt( char *objName, cwbDB_RequestHandle sqlRequest, char objType )
{
    unsigned int             rc = 0;
    char                     stmt[200];          /* SQL statement buffer          */
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error*/
    signed long              hostErrorCode = 0;  /* AS/400 operation return code  */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.   */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.      */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.     */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.    */

    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }


    if ( objType == 'C' )
      sprintf( stmt, "DROP COLLECTION %S", objName );
    else /* Failsafe to the smallest object we can - a file */
      sprintf( stmt, "DROP TABLE %S", objName );

    rc = cwbDB_SetStatementText( sqlRequest, stmt, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Execute the DROP statement to put the data into the file. */
    verbose(">>> Execute the DROP statement \n");
    rc = cwbDB_Execute( sqlRequest, msgHandle );
    if (rc != CWB_OK)
    {
      displayErrors( "cwbDB_Execute",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
    }


  return(rc);

} /* end dropIt */

