/************************************************************************/
/*                       LICENSE AND DISCLAIMER					        */
/*                                                              		*/
/* This material contains IBM copyrighted sample programming source	    */
/* code for your consideration.  This sample code has not been		    */
/* thoroughly tested under all conditions.  IBM, therefore, cannot	    */
/* guarantee or imply reliability, serviceability, or function.		    */
/* IBM provides no program services for this material. This material	*/
/* is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR	*/
/* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	*/
/* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME		    */
/* JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO	*/
/* THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM	    */
/* BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER  	*/
/* CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,		*/
/* WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS    */
/* OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR	    */
/* OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH	    */
/* DAMAGES.


                          COPYRIGHT                              		*/
/*                              ---------                              	*/
/*               (C) Copyright IBM CORP. 1998, 1999                   	*/
/*               All rights reserved.                                  	*/
/*               US Government Users Restricted Rights -               	*/
/*               Use, duplication or disclosure restricted             	*/
/*               by GSA ADP Schedule Contract with IBM Corp.           	*/
/*               Licensed Material - Property of IBM                   	*/
/*******************************************************************
/*******************************************************************
*
* Module:
*   DBSMINS.C
*
* Purpose:
*    Insert using LOB Locators 
*
* Requires:
*    OS/400 R450 or later 
*    CAE R510 
*           Use of conversion true (default) requires R510 Service 
*           Pack 3 or later (P9953041)
*******************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <crtdbg.h>

#include "CWBDB.H"       /* DB API include file     */
#include "CWBSV.H"       /* Sevice API include file */
#include "CWBDT.H"       /* Numeric conversions */
#include "CWBNLCNV.H"	 /* CCSID Conversion */
#include "CWBNL.H"
#include "dbsmdsps.h"

#define SYSLEN  14       /* Maximum system name is 14 plus the null char    */
#define LIBLEN  11       /* Maximum library name is 10 plus the null char   */
#define PKGLEN  11       /* Maximum SQL package name is 10 plus null char.  */
#define FILELEN 11       /* Maximum file name is 10 plus null char.         */
#define CWBDB_CONVERSION CWB_FALSE  /* Controls use of cwbDB Conversion Indicator value */

/* Displays the required parameters to pass to this program. */
void myUsage(void);

/* A running monologue of the program if the program is compiled with */
/* the variable VERBOSE set.                                          */
void verbose( char * myString );


/* Cycles through the parameter markers, calling displayParameterInfo */
/* for each parameter marker and changing packed and zoned numeric    */
/* parameters to PC long variables.                                   */
unsigned int workWithParameterMarkers( cwbDB_FormatHandle pmDataFormat );


/* This routine is used to acquire data for upload. */
unsigned int readData( char *dataBlock,      unsigned long rowSize,
               char *indicatorBlock, unsigned long nbrParms,
			   cwbDB_DataHandle  pmDataHandle);

/* This routine creates a collection (library) on the AS/400. */
unsigned int createCollection( char *libName, cwbDB_RequestHandle sqlRequest );

/* This routine deletes a collection (library) or a table (file) on the AS/400. */
/* Dangerous stuff!                                                             */
/*   objName      = 1-10 character name of collection or table.                 */
/*   sqlRequest   = previously created SQL request handle.                      */
/*   objType      = 'C' for collection, 'T' for table. 'T' is default.          */
unsigned int dropIt( char *objName, cwbDB_RequestHandle sqlRequest, char objType );

/* This routine creates a member in a file. Expects some data to be previously  */
/* stored on the AS/400 in the data buffer associated with the request handle.  */
unsigned int createMember( char *mbrName,  /* Name of member to be created      */
                           char *mbrText,  /* Text desciption of member created */
                           cwbDB_RequestHandle ndbRequestHandle ); /* handle    */

    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.  */
    cwbDB_RequestHandle      ndbRequestHandle,   /* Object manipulation handle.  */
                             sqlRequestHandle;   /* SQL operation handle.        */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.     */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.    */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.   */
    cwbDB_ConnectionHandle   connectionHandle;   /* Handle to enable conneciton. */
    cwbDB_FormatHandle       pmDataFormat;       /* Data format handle for       */
                                                 /* parameter marker data.       */
    char                     sysname[SYSLEN],    /* system name.                 */
                             *sysptr;            /* system name ptr.             */
    unsigned int             rc;                 /* return codes                 */
    int                      i = 0, k = 0;       /* miscellaneous integers       */
    unsigned long            j = 0,
                             parmLength = 0,     /* parameter marker length          */
                             nbrParms = 0;       /* number of parameter markers      */
    unsigned short           parmPrecision = 0,  /* parameter marker precision       */
                             parmScale = 0;      /* parameter marker scale           */
    signed short             columnType = 0;     /* data type of a column            */
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error   */
    signed long              hostErrorCode = 0;  /* AS/400 operation return code     */
    char                     *dataBlock;         /* pointer to data buffer           */
    char                     *indicatorBlock;    /* pointer to null indicator buffer */

int main(int argc, char *argv[])
{

  /* Check on the command line parameters. Expect an AS/400 system name only */
  /* Might get something else or a help request.                             */
    if (argc > 1)
    {
        /* Did user request help? */
        if (*argv[1] == '?' || !strncmp(argv[1], "-h", 2))
        {
            myUsage();
            return(-1);
        }
        else { sysptr = argv[1]; } /* Assume system name given. */
    }
    else { myUsage(); return -1; }

    /* Copy system name from ARGV to local variable  */
    strncpy(sysname, sysptr, SYSLEN);

    /* Create PC error message handle. This handle is used by the API layer      */
    /* to log serviceability messages.                                           */
    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create a handle to enable connection to the AS/400. SQL requests */
    /* and data formats will be tied to a single connection handle.     */
    /*     Think of the connection handle as a collection of associated */
    /*     requests and data formats.                                   */
    /********************************************************************/

    /* Create connection handle to the AS/400 system using the system   */
    /* name passed in to this program.                                  */
    verbose(">>> Create connection handle to the AS/400 system. \n");
    printf("\nCreating connection handle to %s . . .\n", sysptr);
    rc = cwbDB_CreateConnectionHandle( sysptr, &connectionHandle, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateConnectionHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Now that we have a connection handle, we can set any job		    */
    /* attributes.  Most must be set before making the connection.      */
    /********************************************************************/
	
	// Lob threshold
	// Sets the size at which a locator is returned for lobs rather then
	// the actual lob data.  To optimize performance  set this 
	// to the maximum amount of memory you think you can allocate
	// for a data buffer.  
	rc = cwbDB_SetLOBFieldThreshold( 
									connectionHandle,
									0, // thresholdSize = 0 --> always use locator for our example 
									msgHandle); 
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetLOBFieldThreshold failed with ");
        printf("return code %d.\n", rc);
    }

    /********************************************************************/
    /* Now that we have a connection handle, we can start a connection  */
    /* with the AS/400 database server.                                 */
    /********************************************************************/

    /* Start communications between the client and the AS/400. */
    verbose(">>> Start communcation between the client and the AS/400. \n");
    printf("\nAttempting to start the server . . .");
    rc = cwbDB_StartServer( connectionHandle, msgHandle );
    if (rc  != CWB_OK)
    {
        printf("\ncwbDB_StartServer failed with return code %d.\n", rc);
        return(rc);
    }

    printf(" started\n\n");

    /********************************************************************/
    /* Now that we have a connection handle, we can create the request  */
    /* handles and data format handles which will be associated with it.*/
    /********************************************************************/

    /* Create SQL request handle for the subsequent APIs that request SQL */
    /* operations to be performed.                                        */
    verbose(">>> Create SQL request handle for the subsequent APIs that request SQL \n");
    rc = cwbDB_CreateSQLRequestHandle( connectionHandle,
                                       &sqlRequestHandle,
                                       msgHandle         );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateSQLRequestHandle failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* All SQL operations require the cursor name and the statement name.       */
    /****************************************************************************/
    /*   statement name (arbitrary up to 18 character name)                     */
    /*                  The database server uses the statement name             */
    /*                  during the PREPARE function in one of its               */
    /*                  optimization steps.                                     */
    /*   cursor name    (arbitrary up to 18 character name)                     */
    /*                  Think of the cursor as a pointer in the                 */
    /*                  database table which points at a row of data.           */
    /****************************************************************************/
    /* Set the statement text in the SQL request.                               */
    verbose(">>> Set statement text \n");
    rc = cwbDB_SetStatementName( sqlRequestHandle, "STMTONE", msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementName failed with return code %d.\n", rc);
        return(rc);
    }

    /* Set the name of the cursor */
    verbose(">>> Set cursor name \n");
    rc = cwbDB_SetCursorName( sqlRequestHandle, "STFCUR1", msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetCursorName failed with return code %d.\n", rc);
        return(rc);
    }


    /*******************************************************************/
    /* First step here is to create a file to insert data into later.  */
    /*******************************************************************/
	verbose(">>> Set statement text \n");
    rc = cwbDB_SetStatementText( sqlRequestHandle, 
								"create table QGPL/pictest " 
								"(PicKey INTEGER, FN VARCHAR(255), EXT VARCHAR(3), " 
								//"PIC BLOB(7000),  DESC CLOB(7000), " 
								"PIC BLOB(7000000),  DESC CLOB(7000000), " 
								"PRIMARY KEY (Pickey) )", 
								msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );


    /* Store the parameters in the SQL RPB */
    verbose(">>> Store the parameters in the SQL RPB \n");
    rc = cwbDB_StoreRequestParameters( sqlRequestHandle,
                                       msgHandle         );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_StoreRequestParameters",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /*****************************************************************************/
    /* Execute                                                                   */	
    /*****************************************************************************/
    verbose(">>> ExecuteImmediate on the CREATE TABLE QGPL/PICTEST \n");
    rc = cwbDB_ExecuteImmediate( sqlRequestHandle,  msgHandle );    
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_ExecuteImmediate",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
	}
	else
	{
		verbose(">>> Test table QGPL/PICTEST created.");
	}
	printf("\n");
	rc = cwbDB_Close( sqlRequestHandle,  msgHandle );    

   /*****************************************************************************/
   /* Now start on the INSERT statement
   /*****************************************************************************/
    verbose(">>> Set statement text \n");
    rc = cwbDB_SetStatementText( sqlRequestHandle, 
								"INSERT INTO QGPL/PICTEST VALUES(?, ?, ?, ?, ?)   ",
								msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );


    /* Store the parameters in the SQL RPB */
    verbose(">>> Store the parameters in the SQL RPB \n");
    rc = cwbDB_StoreRequestParameters( sqlRequestHandle,
                                       msgHandle         );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_StoreRequestParameters",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }



    /*****************************************************************************/
    /* PREPARE the INSERT statement. This checks the syntax and creates the      */
    /* necessary access links on the AS/400.                                     */
    /*****************************************************************************/
    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    verbose(">>> Prepare the INSERT statement \n");
    rc = cwbDB_Prepare( sqlRequestHandle,  msgHandle );
    if ( rc != CWB_OK )
    {
        displayErrors( "cwbDB_Prepare",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }


  /********************************************************************/
  /* The parameter marker data format provides a way of describing    */
  /* the parameter marker variables. The parameter marker data format */
  /* has information about each column such as the column data type,  */
  /* the column data length, the column name...                       */
  /* The data format also has information about a row of data such    */
  /* as the size of a row of data.                                    */
  /********************************************************************/
  /* Here we are creating the parameter marker data format handle.    */
  /********************************************************************/

  rc = cwbDB_CreateParameterMarkerFormatHandle( connectionHandle,
                                                &pmDataFormat,
                                                msgHandle        );
  if (rc != CWB_OK)
  {
      printf("\ncwbDB_CreateParameterMarkerFormatHandle failed with return code %d.\n", rc);
      return(rc);
  }

  ////////////////////////////////////////////////////////////////////////////
  // When conversion is TRUE it will convert some data such as short and longs
  // from Intel to 400 formats automatically.  When FALSE the application
  // must convert the data.  This sample shows both methods.  The method used
  // is controlled by the CONVERSION #define above.
  //
  // There is an open problem with using TRUE with LOB locator where the LOB
  // locator used on the SetData is corrupted resulting in a "SQL0423 LOB 
  // Locator Not Valid" error on the Execute.
  //  
  // The fix for the problem should be in the 3rd service pack of R510 
  // (the PTF number has not yet been assigned).
  //
  /////////////////////////////////////////////////////////////////////////////
  if (!CWBDB_CONVERSION)
  {
	rc = cwbDB_SetConversionIndicator(pmDataFormat,
                                    CWB_FALSE,
                                    msgHandle );
	  if (rc != CWB_OK)
	  {
		  printf("\ncwbDB_SetConversionIndicator failed with return code %d.\n", rc);
		  return(rc);
	  }
	  // Contrary to the header file doc, you do not need to call ApplyAttributes
	  // since this is a PC side setting.
  }

  //Set up the info for the describe....

  /* Ask for the host info to be returned. */
   verbose(">>> Ask for the host info to be returned. \n");
   rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

  /********************************************************************/
  /* Now we'll ask the AS/400 to return the actual data format        */
  /* based on the results of a describe operation on the INSERT       */
  /* statement parameter markers.                                     */
  /********************************************************************/
  verbose(">>> Ask for the PM data format to be returned. \n");
  rc =  cwbDB_ReturnParameterMarkerFormat( sqlRequestHandle,
                                           pmDataFormat,
                                           msgHandle          );
  if (rc != CWB_OK)
  {
      printf("\ncwbDB_ReturnParameterMarkerFormat failed with return code %d.\n", rc);
      return(rc);
  }

  /*********************************************************************/
  /* Now we'll ask the AS/400 to perform the parameter marker describe */
  /* operation on the INSERT statement.                                */
  /*********************************************************************/
  verbose(">>> DescribeParameterMarkers the INSERT statement \n");
  rc = cwbDB_DescribeParameterMarkers( sqlRequestHandle,
                                    msgHandle         );
  if ( rc != CWB_OK )
  {
      displayErrors( "cwbDB_DescribeParameterMarkers",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
      return(rc);
  }

  //Process the data from the describe....
  /*********************************************************************/
  /* Find out how many parameter markers there were in the INSERT      */
  /* statement.                                                        */
  /*********************************************************************/
  rc = cwbDB_GetParameterCount( pmDataFormat,  (unsigned long*)&nbrParms, msgHandle     );
  if ( rc != CWB_OK )
  { return(rc); }

  /***************************************************************/
  /* This next call is to display info about each parameter      */
  /* and change packed or zoned numerics to a more PC            */
  /* friendly format.                                            */
  /* It's not necessary to the operation of the upload, just     */
  /* an interesting aside.                                       */
  /***************************************************************/
  rc = workWithParameterMarkers( pmDataFormat );
  /***************************************************************/
  /* We now return to the regularly scheduled programming. . .   */
  /***************************************************************/


    /* Get the length of one row of input */
    rc = cwbDB_GetSizeOfInputParameters( pmDataFormat,
										CWBDB_LOCAL,
										&parmLength,
										msgHandle     );
    if ( rc != CWB_OK )
    {
      printf( "\ncwbDB_GetSizeOfInputParameters failed with return code: %d \n", rc );
      return(rc);
    }
	else 
		printf("  SizeOfInputParameters is %d ", parmLength);

	/* THIS NEXT CHUNK YOU'LL PROBABLY WANT TO PUT INTO A LOOP  UNLESS     */
	/* YOU ARE LUCKY ENOUGH TO HAVE ALL OF THE DATA IN ONE BUFFER ALREADY. */

    /*******************************************************************/
    /* allocate a buffer for the non-lob parameter data                */    
    /*******************************************************************/
    dataBlock = (char *) malloc( parmLength );

    /* Always a good idea to clear the memory */
    memset( dataBlock,0,parmLength);

    /**************************************************************************/
    /* Each parameter may possibly be a null capable field, and thus need     */
    /* a nullindicator. Each null indicator is two bytes.                     */
    /* nbrParms * size of null indicator * nbrRows is the                     */
    /* size of the indicator buffer.                                          */
    /**************************************************************************/
    indicatorBlock = (char*) malloc( nbrParms * 2 );

    /* Always a good idea to clear the memory */
    memset( indicatorBlock,0,nbrParms*2);

    /***************************************************************************/
    /* The next routine could be used to read data from a file.                */
    /* For this sample, all it does is generate a rows of data to be inserted. */
	/* The LOBs are written here                                               */
    /***************************************************************************/
	rc = readData( dataBlock, parmLength, indicatorBlock, nbrParms, pmDataFormat );
    if ( CWB_OK != rc )
		return(rc) ;

    /* Set the block of non-lob parameter data including the locators */
    verbose(">>> Set parameter marker block\n");
    rc = cwbDB_SetParameterMarkers( sqlRequestHandle,
                                        //1, 
                                        pmDataFormat,
                                        (void*)dataBlock, 
                                        (signed short *)indicatorBlock,
                                        msgHandle );

    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetParameterMarkerBlock failed. rc: %d ",rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_ReturnHostErrorInfo failed. rc: %d ",rc);
        return(rc);
    }
	

    /* Execute the INSERT statement to put the data into the file. */
    verbose(">>> Execute the INSERT statement \n");
    rc = cwbDB_Execute( sqlRequestHandle,
                        msgHandle         );
    if (rc != CWB_OK)
    {
      displayErrors( "cwbDB_Execute",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
      return(rc);
    }


  /* IF YOU MAKE THIS INTO A LOOP, END THE LOOP HERE. */

    /* End communications between the client and the AS/400. */
    verbose(">>> End communications between the client and the AS/400. \n");
    printf("\nEnding the server . . .");
    if ((rc = cwbDB_StopServer(connectionHandle, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_StopServer failed with return code %d.\n", rc);
        return(rc);
    }


return(rc);

} /* end of main */

/************************************************************************/
void myUsage(void)
{


        printf("\n\n\n\nThis program uses the Client Access/400 DB APIs to INSERT a block\n");
        printf("of data into a database file on the AS/400.\n\n\n\n" );
        printf("\nThis program requires a system name to be passed as a parameter. \n\n\n" );


}
/************************************************************************/
void verbose( char * myString )
{
#if defined(VERBOSE)
  printf("%s",myString);
#endif
}

/************************************************************************/
/* A routine to "read" data in from a file. For simplicity, it just     */
/* creates a little data on its own.                                    */
/************************************************************************/
unsigned int readData( char *dataBlock,      unsigned long rowSize,
               char *indicatorBlock, unsigned long nbrParms,
			   cwbDB_DataHandle		 pmDataFormat)

{
    /*******************************************************************************/
    /* This next chunk is for building my dummy data.
    /*                                                                      <<<<<< */
    /* You would probably want to replace it with code to read              <<<<<< */
    /* from a data file.                                                    <<<<<< */
    /*                                                                      <<<<<< */
    unsigned long            j = 0;         
    unsigned long            i =0;  
	UCHAR *					 pFieldData = 0;
    unsigned long            parmLength = 0;
    signed short             *indicatorPtr;                             /*  <<<<<< */
	unsigned long			 ulParamBLOBLocator;
	unsigned long			 ulParamCLOBLocator;
	UCHAR  *				 pLOBData = 0;
	unsigned long			 ulLOBLen = 0;
	long					 rc;
	cwbDB_SQLCA				 sqlca;


	//conversion stuff 
	long			intData;
	unsigned long	nmbrerrs; 
	unsigned long	posoferr; 
	unsigned long	rqdlen; 

	///////////////
    //PICKEY field
    pFieldData =  dataBlock; 
	if (CWBDB_CONVERSION)
		* (long *) pFieldData = 1;		//Use only if conversion is enabled
	else
	{
		intData =  1;						//Convert to 400 format
		cwbDT_Bin4ToBin4(pFieldData, (char*) &intData );	//Converter to/from must be different
	}

	/////////////////
	//FN
	//Bump pointer to next field
	rc = cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, 1, &parmLength,  0 );
	pFieldData += parmLength;		
	* (short *) pFieldData  = (short) strlen("TESTBLOB.BIN");				//2 byte length of the varchar
	memcpy((char *)pFieldData + 2,"TESTBLOB.bin", strlen("TESTBLOB.BIN"));  //FN 
	if (!CWBDB_CONVERSION)
    {	
		//Convert the string.  
		rc = cwbNL_ConvertCodePages(CWBNL_CP_CLIENT_ANSI, CWBNL_CP_AS400,
									strlen("TESTBLOB.BIN"), strlen("TESTBLOB.BIN"),
								   (char *)pFieldData + 2, (char *)pFieldData + 2,
								   &nmbrerrs, &posoferr, &rqdlen, 0); 

		//Swap the short.  To,from should be different per the doc but it seems to work....
		cwbDT_Bin2ToBin2(pFieldData, pFieldData);	//Convert to 400 format
	}
		
	/////////////////
	//EXT field
	//Bump pointer to next field
	rc = cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, 2, &parmLength,  0 );
    pFieldData += parmLength;		
	*(short *) pFieldData = strlen("BIN");	//2 byte length of the varchar
	memcpy((char *)pFieldData + 2,"bin", strlen("BIN"));  //EXT
	if (!CWBDB_CONVERSION)
	{
	    //Convert the string.  
		rc = cwbNL_ConvertCodePages(CWBNL_CP_CLIENT_ANSI, CWBNL_CP_AS400,
								strlen("BIN"), strlen("BIN"),
								(char *)pFieldData + 2, (char *)pFieldData + 2,
								&nmbrerrs, &posoferr, &rqdlen, 0); 
		//Convert the short
		cwbDT_Bin2ToBin2(pFieldData, pFieldData);	//Convert to 400 format
	}
    

	/////////////////////////////////////////////////////////////////////////////////
	// We can not get the maximum size of the LOB parameter from cwbDB like
	// we can for columns.
	// We are dealing with a LOB parameter and we must know what
	// the locator is first so we can set the entire parameter's LOB data
	// NOTE:  Do not convert or alter the locator value.  
	/////////////////////////////////////////////////////////////////////////////////
	rc = cwbDB_GetLOBLocator( pmDataFormat,		//Parameter format handle
		                        4,				//1 based ordinal
		                        &ulParamBLOBLocator,
		                        msgHandle);
	if (rc != CWB_OK)
	{
		printf("\ncwbDB_GetLOBLocator failed. rc: %d ",rc);
		return rc;
	}
	rc = cwbDB_GetLOBLocator( pmDataFormat,		//Parameter format handle
		                        5,				//1 based ordinal
		                        &ulParamCLOBLocator,
		                        msgHandle);
	if (rc != CWB_OK)
	{
		printf("\ncwbDB_GetLOBLocator failed. rc: %d ",rc);
		return rc;
	}
	
	////////////////////////////////////////////////////////////////
	// First the BLOB field's data
	////////////////////////////////////////////////////////////////
	//Make up some data.  This normally might come from a PC file.
	ulLOBLen = 200;
    pLOBData = (unsigned char*) malloc( ulLOBLen );
	for (i = 0; i < ulLOBLen; i++)
	{
		*(pLOBData + i) = (unsigned char) i % 255;		
	}	
    

	// WriteLOBData does not necessarily need to return any information
	// from the server unless an error occurs.  We won't know about any
	// errors unless we ask the server to return something. 
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_ReturnHostErrorInfo failed. rc: %d ",rc);
        return(rc);
    }
		
	rc = cwbDB_ReturnSQLCA(sqlRequestHandle,	// Request handle
				           &sqlca,				// SQLCA structure
				           0);					// No error handle here

	rc = cwbDB_WriteLOBData(sqlRequestHandle,		// Request handle being used for this cwbDB session
				               pLOBData,			// Pointer to the array of unsigned chars
				               ulParamBLOBLocator,	// Locator we just got above
				               65535,				// Pass the CCSID even though the
				                                    // data base APIs do not use this
				                                    // to do any translation on their own.
				                                    // It is simply put in the structure
				                                    // passed to the host.
				               ulLOBLen,			// The whole thing (always in characters)
				               0,					// Starting position (beginning)
				               msgHandle);			// In case of error
	free(pLOBData);
	if (rc != CWB_OK)
	{
        displayErrors( "\ncwbDB_WriteLOBData",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );

		return rc;
	}

	////////////////////////////////////////////////////////////////
	// Next the CLOB field's data
	////////////////////////////////////////////////////////////////
	//Make up some data.  This normally might come from a PC file.
	ulLOBLen = 200;
    pLOBData = (unsigned char*) malloc( ulLOBLen );
	for (i = 0; i < ulLOBLen; i++)
	{
		*(pLOBData + i) = (unsigned char) (48 + i % 75);	//Some valid ascii characters
	}	
    //Convert it.  There is no automatic conversion available for CLOBS....
	rc = cwbNL_ConvertCodePages(CWBNL_CP_CLIENT_ANSI, CWBNL_CP_AS400, ulLOBLen, 
           ulLOBLen, pLOBData, pLOBData, &nmbrerrs, &posoferr, &rqdlen, 
           0); 

	// WriteLOBData does not necessarily need to return any information
	// from the server unless an error occurs.  We won't know about any
	// errors unless we ask the server to return something. 
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( sqlRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_ReturnHostErrorInfo failed. rc: %d ",rc);
        return(rc);
    }

	rc = cwbDB_WriteLOBData(sqlRequestHandle,		// Request handle being used for this cwbDB session
				               pLOBData,			// Pointer to the array of unsigned chars
				               ulParamCLOBLocator,	// Locator we just got above
				               65535,				// Pass the CCSID even though the
				                                    // data base APIs do not use this
				                                    // to do any translation on their own.
				                                    // It is simply put in the structure
				                                    // passed to the host.
				               ulLOBLen,			// The whole thing (always in characters)
				               0,					// Starting position (beginning)
				               msgHandle);			// In case of error
	free(pLOBData);
	if (rc != CWB_OK)
	{
        displayErrors( "\ncwbDB_WriteLOBData",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
		return rc;
	}

	/////////////////////////////////////////////////////////////////
	// Set the parameter data for the locator fields as the value of
	// the locator received above.  This allows the host 
	// to correlate the data we just got done sending seperatly
	// to this parameter.  Even if conversion is true cwbDB will leave
	// the locator value uncoverted so pass it in the same way it
	// was received.
	/////////////////////////////////////////////////////////////////

	//Bump pointer to next field - the BLOB locator
	cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, 3, &parmLength,  0 );
	pFieldData += parmLength;		
	* (long *) pFieldData = ulParamBLOBLocator;	
		
	//Bump pointer to next field - the CLOB locator
	cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, 4, &parmLength,  0 );
	pFieldData += parmLength;
	* (long *) pFieldData = ulParamCLOBLocator;

	//verify we didn't overshoot
	cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, 5, &parmLength,  0 );
	pFieldData += parmLength;		//should now be at end of buffer	
	if (rowSize != (unsigned long) ( pFieldData - dataBlock))
	{
		printf("\nOverflowed buffer!  SizeOfInputParams %d, Total size %d",
				rowSize, (long) (dataBlock - pFieldData));
		return -1;
	}

    /*  End of building dummy data                                          <<<<<< */
    /*                                                                      <<<<<< */
    /* Now it's necessary to build a set of null indicators  for the data.  <<<<<< */
    /* Each indicator is two bytes, and there must be one for every field   <<<<<< */
    /* in every row of data to be inserted (#rows*#columns*2)               <<<<<< */
    /* 0 = field is not null, -1 == field is null                           <<<<<< */
    /*                                                                      <<<<<< */
    indicatorPtr = (unsigned short*)indicatorBlock;                     /*  <<<<<< */
    for ( i = 1; i <=1; i++)  //for each row                            /*  <<<<<< */
    {                                                                   /*  <<<<<< */
      for ( j = 0; j < nbrParms; j++ ) // for each param                /*  <<<<<< */
      {                                                                 /*  <<<<<< */
        *indicatorPtr = 0;                                              /*  <<<<<< */
        indicatorPtr += 1;                                              /*  <<<<<< */
      }                                                                 /*  <<<<<< */
    }                                                                   /*  <<<<<< */
    /*******************************************************************************/
	return CWB_OK;
} /* end readData */



unsigned int createCollection( char *libName, cwbDB_RequestHandle sqlRequest )
{
    unsigned int             rc = 0;
    char                     stmt[200];          /* SQL statement buffer          */
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error*/
    signed long              hostErrorCode = 0;  /* AS/400 operation return code  */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.   */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.      */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.     */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.    */

    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    sprintf( stmt, "CREATE COLLECTION %S", libName );

    rc = cwbDB_SetStatementText( sqlRequest, stmt, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Execute the CREATE COLLECTION statement to put the data into the file. */
    verbose(">>> Execute the CREATE statement \n");
    rc = cwbDB_Execute( sqlRequest, msgHandle );
    if (rc != CWB_OK)
    {
      displayErrors( "cwbDB_Execute",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
    }


  return(rc);

}  /* end createCollection  */

/*******************************************************************************/
/* This routine creates a member in a file. Expects some data to be previously */
/* stored on the AS/400 in the data buffer associated with the request handle. */
/*******************************************************************************/
unsigned int createMember( char *mbrName, char *mbrText, cwbDB_RequestHandle ndbRequestHandle )
{
    unsigned int             rc = 0;
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error*/
    signed long              hostErrorCode = 0;  /* AS/400 operation return code  */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.   */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.      */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.     */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.    */

    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /***************************************************************************/
    /* Now we are going to create a second member in the source physical file. */
    /***************************************************************************/
    /* Set the _description_ for the next DB file member to create             */
    verbose(">>> Set the description for the next DB file member created \n");
    rc = cwbDB_SetMemberText( ndbRequestHandle, mbrText, msgHandle  );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetMemberText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Set the _name_ for the next DB file member created */
    verbose(">>> Set the name for the next DB file member created \n");
    rc = cwbDB_SetMemberName( ndbRequestHandle, mbrName,  msgHandle  );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetMemberName failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Ask for the host info to be returned. */
    verbose(">>> Ask for the host info to be returned. \n");
    rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
                                     &hostErrorClass,
                                     &hostErrorCode,
                                     hostMsgID,
                                     firstLevelMessage,
                                     secondLevelMessage,
                                     msgHandle                );

    /***********************************************************************/
    /* Use the parameters stored in the RPB along with the new parameters  */
    /* just set to perform the next operation.                             */
    /***********************************************************************/
    /* Create the next member                                              */
    /***********************************************************************/
    verbose(">>> Create the next member \n");
    rc = cwbDB_AddMember( ndbRequestHandle,  msgHandle  );
    if (rc != CWB_OK)
    {
        displayErrors( "cwbDB_AddMember",
                       rc,        hostErrorClass,    hostErrorCode,
                       hostMsgID, firstLevelMessage, secondLevelMessage );
        return(rc);
    }

  return(rc);
}   /* end createMember */


/****************************************************************/
/* This routine steps through the parameter marker data format, */
/* displaying information about each parameter marker and       */
/* changing zoned and packed numerics to PC longs for           */
/* convenience.                                                 */
/****************************************************************/
unsigned int workWithParameterMarkers( cwbDB_FormatHandle pmDataFormat )
{

    unsigned int             rc;                 /* return codes                     */
    int                      k = 0;              /* miscellaneous integers           */
    unsigned long            i= 0, j = 0,
                             parmLength = 0,     /* parameter marker length          */
                             nbrParms = 0;       /* number of parameter markers      */
    unsigned short           parmPrecision = 0,  /* parameter marker precision       */
                             parmScale = 0;      /* parameter marker scale           */
    signed short             columnType = 0;     /* data type of a column            */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.      */


    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /*********************************************************************/
    /* Find out how many parameter markers there were in the INSERT      */
    /* statement. For this exercise, it will return 3. The ? before the  */
    /* ROWS clause is not included in this count.                        */
    /*********************************************************************/
    rc = cwbDB_GetParameterCount( pmDataFormat,  (unsigned long*)&nbrParms, msgHandle     );
    if ( rc != CWB_OK )
    { return(rc); }


    j = nbrParms;

    /***********************************************************/
    /* This next loop is to display info about each parameter  */
    /* and change packed or zoned numerics to a more PC        */
    /* friendly format.                                        */
    /* It's not necessary to the operation of the upload, just */
    /* an interesting aside.                                   */
    /***********************************************************/

    for (i = 1; i <= j ;i++ )
    {

      /* Get the PC data type of the parameter  */
      rc = cwbDB_GetParameterType( pmDataFormat, CWBDB_LOCAL, i, &columnType, msgHandle     );
      if ( rc != CWB_OK )
      {
        printf( "\ncwbDB_GetParameterType failed for parameter marker %d with return code: %d \n", i, rc );
        return(rc);
      }

      /* Get the length of the parameter */
      rc = cwbDB_GetParameterLength( pmDataFormat, CWBDB_LOCAL, i, &parmLength,  msgHandle     );
      if ( rc != CWB_OK )
      {
        printf( "\ncwbDB_GetParameterLength failed for parameter marker %d with return code: %d \n", i, rc );
        return(rc);
      }


     /* Massage the packed and zoned numerics. */
     if (  ( columnType == CWBDB_PCPACKED )
        || ( columnType == CWBDB_PCZONED  ) )
        { /* Is it a packed or zoned numeric?         */
          /* If so, what are the precision and scale? */
          /* We'll need them to convert parameter to  */
          /* PC LONG type for our convenience.        */
          rc = cwbDB_GetParameterPrecision( pmDataFormat, CWBDB_LOCAL, i, &parmPrecision, msgHandle );
          if ( rc != CWB_OK )
          {
            printf( "\ncwbDB_GetParameterPrecision failed for parameter marker %d with return code: %d \n", i, rc );
            return(rc);
          }

          rc = cwbDB_GetParameterScale( pmDataFormat, CWBDB_LOCAL, i, &parmScale, msgHandle );
          if ( rc != CWB_OK )
          {
            printf( "\ncwbDB_GetParameterScale failed for parameter marker %d with return code: %d \n", i, rc );
            return(rc);
          }

          /* It's easier to use a LONG on the PC, so change the type. */
          rc = cwbDB_SetClientParameterToNumeric( pmDataFormat,
                                                  i,
                                                  CWBDB_PCLONG,
                                                  parmLength,
                                                  parmPrecision,
                                                  parmScale,
                                                  msgHandle );
          if ( rc != CWB_OK )
          {
            printf( "\ncwbDB_SetClientParameterToNumeric failed for parameter marker %d with return code: %d \n", i, rc );
            return(rc);
          }
          columnType = CWBDB_PCLONG;

        }

      displayParameterInfo( i, columnType, parmLength );

    } /* endfor */

  return(rc);

} /* end workWithParameterMarkers */


unsigned int dropIt( char *objName, cwbDB_RequestHandle sqlRequest, char objType )
{
    unsigned int             rc = 0;
    char                     stmt[200];          /* SQL statement buffer          */
    unsigned short           hostErrorClass = 0; /* Classification of AS/400 error*/
    signed long              hostErrorCode = 0;  /* AS/400 operation return code  */
    cwbSV_ErrHandle          msgHandle;          /* Handle for service logging.   */
    cwbDB_DataHandle         hostMsgID;          /* Host message identifier.      */
    cwbDB_DataHandle         firstLevelMessage;  /* First level message text.     */
    cwbDB_DataHandle         secondLevelMessage; /* Second level message text.    */

    verbose(">>> Create serviceability error handle. \n");
    rc = cwbSV_CreateErrHandle( &msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /********************************************************************/
    /* Create 3 data handles used for returning host error information. */
    /********************************************************************/
    /* Create a data handle for the host message id. */
    verbose(">>> Create a data handle for the host message id. \n");
    rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the first level message text. */
    verbose(">>> Create a data handle for the first level message text. \n");
    rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }

    /* Create a data handle for the second level message text. */
    verbose(">>> Create a data handle for the second level message text. \n");
    rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        return(rc);
    }


    if ( objType == 'C' )
      sprintf( stmt, "DROP COLLECTION %S", objName );
    else /* Failsafe to the smallest object we can - a file */
      sprintf( stmt, "DROP TABLE %S", objName );

    rc = cwbDB_SetStatementText( sqlRequest, stmt, msgHandle );
    if (rc != CWB_OK)
    {
        printf("\ncwbDB_SetStatementText failed ");
        printf("with return code %d.\n", rc);
        return(rc);
    }

    /* Execute the DROP statement to put the data into the file. */
    verbose(">>> Execute the DROP statement \n");
    rc = cwbDB_Execute( sqlRequest, msgHandle );
    if (rc != CWB_OK)
    {
      displayErrors( "cwbDB_Execute",
                     rc,        hostErrorClass,    hostErrorCode,
                     hostMsgID, firstLevelMessage, secondLevelMessage );
    }


  return(rc);

} /* end dropIt */

