/************************************************************************/
/*                       LICENSE AND DISCLAIMER					                */
/*                                                              		    */
/* This material contains IBM copyrighted sample programming source	    */
/* code for your consideration.  This sample code has not been		      */
/* thoroughly tested under all conditions.  IBM, therefore, cannot	    */
/* guarantee or imply reliability, serviceability, or function.		      */
/* IBM provides no program services for this material. This material	  */
/* is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR	*/
/* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	  */
/* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME		      */
/* JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO	  */
/* THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM	    */
/* BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER  	*/
/* CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,		    */
/* WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS    */
/* OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR	    */
/* OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH	    */
/* DAMAGES.


                          COPYRIGHT                              		    */
/*                              ---------                              	*/
/*               (C) Copyright IBM CORP. 1998, 1999                   	*/
/*               All rights reserved.                                  	*/
/*               US Government Users Restricted Rights -               	*/
/*               Use, duplication or disclosure restricted             	*/
/*               by GSA ADP Schedule Contract with IBM Corp.           	*/
/*               Licensed Material - Property of IBM                   	*/
/*******************************************************************
/*******************************************************************
*
* Module: DBSMLIBL.C
*
* Purpose:
*	 Shows how to add a library to the user's AS/400 library list for
*	 a given connection.
*
*  In addition it shows how get get a library list and step through it
*  and also how to look for a libary on the system.
*******************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "CWBDB.H"
#include "CWBSV.H"

#include "dbsmdsps.h"
#define SYSLEN	14		 /* Maximum system name is 14 plus the null char  */
#define LIBLEN	11		 /* Maximum library name is 10 plus the null char */
#define PKGLEN	11		 /* Maximum SQL package name is 10 plus null char. */
#define FILELEN 11		 /* Maximum file name is 10 plus null char. */

cwbDB_ConnectionHandle	 connectionHandle;

// local functions
void myUsage(void);
void SevereErr(char* api, unsigned int rc);
int AlreadyInLibraryList(char * l);
int LibraryExists(char * libName);
void GetAndDisplayError(char * msg, cwbDB_RequestHandle Rhandle, unsigned int rc2);
int GetLibraryName( char * libName);

int main(int argc, char *argv[])
{
	cwbSV_ErrHandle 		msgHandle;
	cwbDB_RequestHandle ndbRequestHandle;
	cwbDB_DataHandle		hostMsgID; 		 /* Host message identifier. */
	cwbDB_DataHandle		firstLevelMessage;  /* First level message text. */
	cwbDB_DataHandle		secondLevelMessage; /* Second level message text. */
	unsigned short			hostErrorClass = 0;
	signed long 				hostErrorCode = 0;	
	char								sysname[SYSLEN],
											libName[31],
											*sysptr;
	unsigned int				rc = 0;
	unsigned int				i = 0;
	
	if (argc > 1)
	{
		/* Did user request help? */
		if (*argv[1] == '?' || !strncmp(argv[1], "-h", 2))
		{
			myUsage();
			return(-1);
		}
		else { sysptr = argv[1]; } /* Assume system name given. */
	}
	
	strncpy(sysname, sysptr, SYSLEN);
	
	/* Create error handle. */
	rc = cwbSV_CreateErrHandle( &msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateErrHandle", rc);
	
	/* Create connection handle to the AS/400 system. */
	printf("\nCreating connection handle to %s . . .\n", sysptr);
	rc = cwbDB_CreateConnectionHandle( sysptr, &connectionHandle, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateConnectionHandle", rc);
	
	/* Start communcation between the client and the AS/400. */
	printf("\nAttempting to start the server . . .\n");
	rc = cwbDB_StartServer( connectionHandle, msgHandle );
	if (rc	!= CWB_OK)
		SevereErr("cwbDB_StartServer", rc);
	
	/* Create NDB request handle for subsequest API calls that request */
	/* operations to be performed with AS/400 file objects. 		   */
	rc=cwbDB_CreateNDBRequestHandle( connectionHandle,
		&ndbRequestHandle,
		msgHandle);
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateNDBRequestHandle", rc);
	
	//Create the handles to get error information
	rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
		
	/*******************************************************************/
	/* Set up thus far...											   */
	/*******************************************************************/
	
	//Prompt for library name and add it to the library list until asked to quit
	while (GetLibraryName(libName))
	{
		/*******************************************************************/
		/* Tell the request handle where to add the library to the library */
		/* list and what library to add.                                   */
		/*******************************************************************/
		rc = cwbDB_SetAddLibraryPosition( ndbRequestHandle,
			CWBDB_ADD_LIBRARY_TO_FRONT ,
			0 );
		if (rc != CWB_OK)
			SevereErr("cwbDB_SetAddLibraryPosition", rc);
		
		rc = cwbDB_SetAddLibraryName( ndbRequestHandle,
			libName, 0 );
		if (rc != CWB_OK)
			SevereErr("cwbDB_SetAddLibraryName", rc);
		
		/*******************************************************************/
		/* Now that the request handle knows the library name and where to */
		/* add it we can tell it do the flow the the host that does the    */
		/* actual add.  We must ask fot the error info to be returned here */
		/* because if we don't then nothing will get returned from the host*/
		/*including the return code.                                       */
		/*******************************************************************/
		rc =  cwbDB_ReturnHostErrorInfo( ndbRequestHandle,
			&hostErrorClass,
			&hostErrorCode,
			hostMsgID,
			firstLevelMessage,
			secondLevelMessage,
			msgHandle				 );
		if (rc != CWB_OK)
			SevereErr("cwbDB_ReturnHostErrorInfo", rc);
		
		rc = cwbDB_AddLibraryToList( ndbRequestHandle, msgHandle );
		
		if (rc != CWB_OK)
		{
			/****************************************************************/
			/* if the add fails the most likely causes are that it is       */
			/* already in the list or the library does not exsist.  If      */
			/* neither of these where the cause then there must have been   */
			/* a more severe error.                                         */
			/****************************************************************/
			if(AlreadyInLibraryList(libName))
				printf("\tLibrary: '%s' is already in the library list. \n\n", libName );
			else
				if(!LibraryExists(libName))
					printf("\tLibrary: '%s' does not exist. \n\n", libName );
				else
					//must be a real error, so print it out and quit.
				{
					displayErrors( "cwbDB_AddLibraryToList",
						rc, hostErrorClass, hostErrorCode,
						hostMsgID, firstLevelMessage, secondLevelMessage );
					exit(rc);
				}
		}
		else
			printf("\tLibrary: '%s' added to the library list. \n\n", libName);
	}

	/********************************************************************
	* Delete all of the handles.  This is probably not necessary when the 
	* program is going to end, but it always best to clean up.
	*********************************************************************/
	rc = cwbDB_StopServer( connectionHandle, msgHandle );
	if (rc	!= CWB_OK)
		SevereErr("cwbDB_StopServer", rc);

	rc=cwbDB_DeleteNDBRequestHandle( ndbRequestHandle, msgHandle);
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteNDBRequestHandle", rc);
		
	rc = cwbDB_DeleteConnectionHandle( connectionHandle, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteConnectionHandle", rc);
		
	rc = cwbDB_DeleteDataHandle( hostMsgID, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( firstLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( secondLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);

	rc = cwbSV_DeleteErrHandle( msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_DeleteErrHandle", rc);
	
	return(rc);
} /* end of main */

/********************************************************************/
/* Go to the host to see if the library exsists.                    */
/* Return true if it does, else false																*/
/********************************************************************/
int LibraryExists(char * libName)
{
	cwbSV_ErrHandle 			msgHandle;
	cwbDB_RequestHandle		catRequestHandle;
	cwbDB_DataHandle			dataBuffer;
	cwbDB_FormatHandle		dataFormat;
	cwbDB_DataHandle			indicatorsBuffer;
	unsigned int					rc = 0;
	int 									i = 0;
	unsigned long					dataLength = 0;
	int										result = 0;
	
	// Need a catalog request handle to check for lib.
	rc = cwbDB_CreateCatalogRequestHandle( connectionHandle,
		&catRequestHandle, msgHandle);
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateCatalogRequestHandle", rc);
	
	/* Create error and data handles. */
	rc = cwbSV_CreateErrHandle( &msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateErrHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataFormatHandle( connectionHandle,
		&dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataFormatHandle", rc);
	
	// Tell the request handle what library we are looking for.
	rc = cwbDB_SetLibraryName( catRequestHandle, libName, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_SetLibraryName", rc);
	
	// Ask the request handle to return the data
	rc =  cwbDB_ReturnData( catRequestHandle,
		dataBuffer,
		indicatorsBuffer,
		dataFormat,
		msgHandle	);
	if (rc != CWB_OK)
		SevereErr("cwbDB_ReturnData", rc);
	
	// Now have the request handle go to the host to look for the
	// library name.
	rc = cwbDB_RetrieveLibraryInformation( catRequestHandle,
		CWBDB_GET_LIBRARY_NAME,
		msgHandle );
	if (rc != CWB_OK)
		GetAndDisplayError("cwbDB_RetrieveLibraryInformation ",
		catRequestHandle, rc);
	
	//data length will be zero id the library was not found
	rc = cwbDB_GetDataLength( dataBuffer, &dataLength, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_GetDataLength", rc);
	if ( 0 != dataLength )
		result = 1;			//library exists
	else 
		result = 0;			//library does not exist

	/****************************************************************
	* Must be sure to delete all of the handles here, otherwise any
	* data the have will be orphaned when the function ends.
	****************************************************************/
	rc = cwbDB_DeleteCatalogRequestHandle(catRequestHandle, msgHandle);
	if (rc != CWB_OK)
		SevereErr("cwbSV_DeleteCatalogRequestHandle", rc);
	
	rc = cwbSV_DeleteErrHandle( msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_DeleteErrHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataFormatHandle( dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataFormatHandle", rc);
	return(result);
}

/********************************************************************/
/* Go to the host to see if the library is already in the list			*/
/* Return true if it is, else false																	*/
/********************************************************************/
int AlreadyInLibraryList(char * libName)
{
	cwbSV_ErrHandle 		msgHandle;
	cwbDB_RequestHandle catRequestHandle;
	cwbDB_DataHandle		dataBuffer;
	cwbDB_FormatHandle	dataFormat;
	cwbDB_DataHandle		indicatorsBuffer;
	unsigned int				rc = 0;
	int 								i = 0;
	unsigned long				j = 0, parmLength = 0, dataLength = 0;
	char								*myData;
	char								buf[40];
	int                 result = 0;
	
	// Need a catalog request handle to get the lib list.
	rc = cwbDB_CreateCatalogRequestHandle( connectionHandle,
		&catRequestHandle,
		msgHandle		  );
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateErrHandle", rc);
	
	/* Create error and data handles. */
	rc = cwbSV_CreateErrHandle( &msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateErrHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataFormatHandle( connectionHandle,
		&dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	// Ask the request handle to return data
	rc =  cwbDB_ReturnData( catRequestHandle,
		dataBuffer,
		indicatorsBuffer,
		dataFormat,
		msgHandle	);
	if (rc != CWB_OK)
		SevereErr("cwbDB_ReturnData", rc);
	
	//Go to the host to get the library list.
	//Because we did not set a library name it will use *USRLIBL
	rc = cwbDB_RetrieveLibraryInformation( catRequestHandle,
		CWBDB_GET_LIBRARY_NAME,
		msgHandle );
	if (rc != CWB_OK)
		GetAndDisplayError("cwbDB_RetrieveLibraryInformation ",
		catRequestHandle, rc);
	
	// ok, now we have the library list, so get a pointer to
	// it, the total length and length of each entry. 
	rc = cwbDB_GetDataPointer( dataBuffer, &myData, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_GetDataPointer", rc);
	
	rc = cwbDB_GetDataLength( dataBuffer, &dataLength, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_GetDataLength", rc);
	
	rc = cwbDB_GetColumnLength( dataFormat, CWBDB_LOCAL, 1, &parmLength, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_GetDataLength", rc);
	
	//step thru the lib list to see if it includes the lib
	memset(buf, ' ', sizeof(buf));
	memcpy(buf, libName, strlen(libName));
	for ( j = 1; j <= dataLength / parmLength; j++ )
	{
		if (0 == memcmp(buf, myData, parmLength))
		{
			result = 1;
			break;
		}
		else { myData += parmLength; }
	}

	/****************************************************************
	* Must be sure to delete all of the handles here, otherwise any
	* data they have will be orphaned when the function ends.
	****************************************************************/
	rc = cwbDB_DeleteCatalogRequestHandle(catRequestHandle, msgHandle);
	if (rc != CWB_OK)
		SevereErr("cwbSV_DeleteCatalogRequestHandle", rc);
	
	rc = cwbSV_DeleteErrHandle( msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_DeleteErrHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( dataBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataHandle( indicatorsBuffer, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataHandle", rc);
	
	rc = cwbDB_DeleteDataFormatHandle( dataFormat, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_DeleteDataFormatHandle", rc);
	return(result);
}

/********************************************************************/
/* When we do not ask for the error information to be returned		  */
/* and there is an error which we would like information about	   	*/
/* then we must ask for the error information and do a get data   	*/
/* to cause a flow the the host to get the error information.	    	*/
/********************************************************************/
void GetAndDisplayError(char * msg, cwbDB_RequestHandle Rhandle, unsigned int rc2)
{
	cwbSV_ErrHandle 		 msgHandle;
	cwbDB_DataHandle		 hostMsgID; 		 /* Host message identifier. */
	cwbDB_DataHandle		 firstLevelMessage;  /* First level message text. */
	cwbDB_DataHandle		 secondLevelMessage; /* Second level message text. */
	unsigned int			 rc = 0;
	unsigned short			 hostErrorClass = 0;
	signed long 			 hostErrorCode = 0;
	
	/* Create error handles. */
	rc = cwbSV_CreateErrHandle( &msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbSV_CreateErrHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &hostMsgID, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &firstLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	rc = cwbDB_CreateDataHandle( &secondLevelMessage, msgHandle );
	if (rc != CWB_OK)
		SevereErr("cwbDB_CreateDataHandle", rc);
	
	
	// Ask for error information and get data
	rc =  cwbDB_ReturnHostErrorInfo( Rhandle,
		&hostErrorClass,
		&hostErrorCode,
		hostMsgID,
		firstLevelMessage,
		secondLevelMessage,
		msgHandle				 );
	if (rc != CWB_OK)
		SevereErr("cwbDB_ReturnHostErrorInfo", rc);
	
	// this will cause a flow to the host which will retreive the
	// error information fot the request handle.
	rc = cwbDB_GetData( Rhandle, msgHandle );
	if (rc != CWB_OK)
		   SevereErr("cwbDB_ReturnHostErrorInfo", rc);
	else
		displayErrors( msg, rc2, hostErrorClass, hostErrorCode,
			hostMsgID, firstLevelMessage, secondLevelMessage );
	exit(rc2);
}

/********************************************************************/
/* An API got an error that will keep the application from running	*/
/* so print the API name and return code and stop execution.		*/
/*																	*/
/********************************************************************/
void SevereErr(char* api, unsigned int rc)
{
	printf("\n%s failed with return code %d.\n", api, rc);
	exit(rc);
}

/************************************************************************/
void myUsage(void)
{
	printf("\n\nMust pass a system name.               \n  \n  \n          \n");
}

/************************************************************************/
/* Prompt for a library name and convert to upper case. 				*/
/* No check is made for overflowing the space at libName.				*/
/* Return true if input is not quit character, else return false.		*/
/************************************************************************/
int GetLibraryName( char * libName)
{
	unsigned int i; 
	
	printf("Enter a library name or 'x' to quit: ");
	scanf("%s",libName);
	i = strlen( libName );
	for ( i = 0; i < strlen( libName ) ; i++ )
		libName[i] = (char)toupper(libName[i]);
	printf("\n\n");
	if( (strlen(libName) == 1) && (libName[0] == 'X') )
		return(0);				//got lib name
	else
		return(1);				//exit
}
