/************************************************************************/
/*                       LICENSE AND DISCLAIMER					                */
/*                                                              		    */
/* This material contains IBM copyrighted sample programming source	    */
/* code for your consideration.  This sample code has not been		      */
/* thoroughly tested under all conditions.  IBM, therefore, cannot	    */
/* guarantee or imply reliability, serviceability, or function.		      */
/* IBM provides no program services for this material. This material	  */
/* is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR	*/
/* IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF	  */
/* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME		      */
/* JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO	  */
/* THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM	    */
/* BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER  	*/
/* CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,		    */
/* WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS    */
/* OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR	    */
/* OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH	    */
/* DAMAGES.


                          COPYRIGHT                              		    */
/*                              ---------                              	*/
/*               (C) Copyright IBM CORP. 1996, 1999                   	*/
/*               All rights reserved.                                  	*/
/*               US Government Users Restricted Rights -               	*/
/*               Use, duplication or disclosure restricted             	*/
/*               by GSA ADP Schedule Contract with IBM Corp.           	*/
/*               Licensed Material - Property of IBM                   	*/

/*
* Module:
*   SMPLSQL.C
*
* Purpose:
*
*   This program demonstrates the use of the following Client Access
*   Optimized SQL API functions to perform some typical Catalog requests,
*   Native DataBase (NDB) requests, and SQL requests.
*
*   APIs used to control attributes of the database server:
*       cwbDB_CreateConnectionHandle
*       cwbDB_DeleteConnectionHandle
*       cwbDB_StartServer
*       cwbDB_StopServer
*
*   APIs for Catalog requests:
*       cwbDB_CreateCatalogRequestHandle
*       cwbDB_DeleteCatalogRequestHandle
*       cwbDB_RetrieveFileInformation
*       cwbDB_RetrieveLibraryInformation
*       cwbDB_RetrieveSQLPackageInformation
*
*   APIs for NDB requests:
*       cwbDB_CreateDuplicateFile
*       cwbDB_CreateNDBRequestHandle
*       cwbDB_CreateSourcePhysicalFile
*       cwbDB_DeleteFile
*       cwbDB_DeleteNDBRequestHandle
*       cwbDB_SetBaseFile
*
*   APIs for SQL requests:
*       cwbDB_CreateSQLRequestHandle
*       cwbDB_DeleteSQLRequestHandle
*       cwbDB_Fetch
*       cwbDB_Open
*       cwbDB_Prepare
*       cwbDB_SetBlockCount
*       cwbDB_SetCursorName
*       cwbDB_SetStatementName
*       cwbDB_SetStatementText
*
*   APIs used with multiple requests types:
*       cwbDB_GetData
*       cwbDB_ReturnData
*       cwbDB_ReturnDataFormat
*       cwbDB_ReturnHostErrorInfo
*       cwbDB_SetFileName
*       cwbDB_SetLibraryName
*       cwbDB_SetPackageName
*       cwbDB_StoreRequestParameters
*
*   APIs used for manipulating the description of data being exchanged
*   between the client and server:
*       cwbDB_CreateDataFormatHandle
*       cwbDB_CreateDataHandle
*       cwbDB_DeleteDataFormathandle
*       cwbDB_DeleteDataHandle
*       cwbDB_GetColumnCount
*       cwbDB_GetColumnLength
*       cwbDB_GetColumnName
*       cwbDB_GetColumnType
*       cwbDB_GetDataLength
*       cwbDB_GetDataPointer
*       cwbDB_GetRowSize
*       cwbDB_SetClientColumnToNumeric
*
*
*   The user is prompted for a choice of requests to perform.  For Catalog
*   requests, this program supports the retrieval of informations on libraries,
*   SQL packages, and files.  For NDB, the creation of either a source physical
*   file or a duplicate file, and the deletion of a file can be requested.
*   For SQL requests, this program demonstrates the execution
*   of SELECT type statements.  Results from both
*   the Catalog and SQL requests are parsed and displayed according to
*   the format informations returned with the data.  When a function is
*   performed on the host server, host error messages
*   are returned and displayed as well.
*
*   Parameters required for the API functions, if not set to default values within
*   the APIs, are prompted from the user interactively.
*
* Usage Notes:
*   The command syntax is:
*
*   SMPLSQL [system_name]
*     where
*       system_name = AS/400 system to run commands on
*
*   Link with CWBAPI.LIB
*
*   This program was tested with Microsoft Visual C++ Version 2.2.
*   Use smplsql.mak to build smplsql.exe.
*
*******************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "CWBDB.H"
#include "CWBSV.H"

#define SMP_OK      0
#define SMP_ERR     1
#define SMP_QUIT    2

#define SYSLEN 14        /* Maximum system name is 14 plus the null char  */
#define LIBLEN 11        /* Maximum library name is 10 plus the null char */

#define BUFLEN  25
#define PKGLEN   11      /* Maximum SQL package name is 10 plus null char. */
#define FILELEN   11     /* Maximum file name is 10 plus null char. */
#define SQLTXTLEN  32766 /* Max SQL statement text is 32765 plus null char. */
#define CURNAMLEN   19   /* Maximum cursor name is 18 plus null char. */
#define SQLNAMLEN   19   /* Maximum SQL statement name is 18 plus null char. */


/*******************************************************************
*
* Function: usage
*
* Purpose:  Display usage notes to user
*
*******************************************************************/
static void usage(void)
{
    printf("usage: SMPLSQL [system_name]\n");
    printf("    where\n");
    printf("\tsystem_name  = AS/400 system to run commands on\n");
}

/******************************************************************
*  Declaration of global variables.
*******************************************************************/
cwbDB_ConnectionHandle  connectionHandle;     /* Handle to an AS/400 server. */
unsigned short      hostErrorClass; /* Indicate which database module has error. */
signed long         hostErrorCode;  /* Return code from the server module. */
cwbDB_DataHandle    hostMsgID;  /* Host message identifier. */
cwbDB_DataHandle    firstLevelMessage; /* First level message text. */
cwbDB_DataHandle    secondLevelMessage; /* Second level message text. */
struct column {                             /* Column structure that includes: */
                 char *     name;                 /*   column name, */
                 unsigned long      nameLength;  /*   column name length, */
                 unsigned long      columnLength; /*  column data length, */
                 signed short       type;           /*  column data type,   */
                 unsigned long      width;  /*   column width in display table. */
};


/******************************************************************
*  Function prototypes
******************************************************************/
void CatalogRequest(void);
void NDBRequest(void);
void SQLRequest(void);
void RetrieveLibrary(cwbDB_RequestHandle requestHandle);
void RetrieveSQLPackage(cwbDB_RequestHandle requestHandle);
void RetrieveFile(cwbDB_RequestHandle requestHandle);
void ReturnData(cwbDB_RequestHandle requestHandle);
void DisplayData(cwbDB_DataHandle data, cwbDB_DataHandle indicator,
                   cwbDB_FormatHandle format, unsigned long  columnCount);
void PrintData(unsigned long  totalWidth, unsigned long  columnCount,
                unsigned long  rowCount, struct column  columnInfo[],
                char *  dataBuffer, unsigned long  rowSize, char *  indicatorBuffer);
void DisplayHostError(cwbDB_RequestHandle  requestHandle);



void strtoupper(char *s)
{
    while (*s) {
        *s = toupper(*s);
        ++s;
    }
}



void resolveErr(cwbSV_ErrHandle errHandle)
{
    static char buffer[BUFSIZ];
    unsigned long errCount, retLen, index;
    unsigned int rc;

    /* If the API failed, it returns an error handle. We can use this handle to call
     * cwbSV_GetErrTextIndexed() Service API and obtain additional error information.
     */
    cwbSV_GetErrCount(errHandle, &errCount);
    for (index = 1; index <= errCount; index++) {
        rc = cwbSV_GetErrTextIndexed(errHandle, index, buffer, (unsigned long) BUFSIZ, &retLen);
        if (rc == CWB_OK || rc == CWB_BUFFER_OVERFLOW)
            printf("%s\n", buffer);
        else if (rc != CWBSV_NO_ERROR_MESSAGES) {
            printf("cwbSV_GetErrTextIndexed(): Service API failed with return code %d.\n", rc);
            break;
        }
    }
}



int main(int argc, char *argv[])
{
    cwbSV_ErrHandle msgHandle;
    char    buffer[BUFLEN], sysname[SYSLEN], *sysptr;
    unsigned int    rc;
    char       input[BUFLEN];

    if (argc > 1) {
        /* Did user request help? */
        if (*argv[1] == '?' || !strncmp(argv[1], "-h", 2)) {
            usage();
            return(SMP_OK);
        }
        else    /* Assume system name given. */
            sysptr = argv[1];
    }
    else  {     /* Prompt user for system name. */
        printf("\nEnter host AS/400 name: ");
        fflush(stdout);
        fgets(buffer, BUFLEN, stdin);
        *(buffer+strlen(buffer)-1) = '\0';
        sysptr = buffer;
    }

    strtoupper(sysptr);
    strncpy(sysname, sysptr, SYSLEN);

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return(SMP_ERR);
    }

    /* Create connection handle to the AS/400 system. */
    printf("\nCreating connection handle to %s . . .", sysptr);
    rc = cwbDB_CreateConnectionHandle(sysptr, &connectionHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateConnectionHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }

    /* Start communcation between the client and the AS/400. */
    printf("\nAttempting to start the server . . .");
    if ((rc = cwbDB_StartServer(connectionHandle, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_StartServer failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }

    /* Create 3 data handles used for returning host error information. */

    /* Create a data handle for the host message id. */
    rc = cwbDB_CreateDataHandle(&hostMsgID, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }

    /* Create a data handle for the first level message text. */
    rc = cwbDB_CreateDataHandle(&firstLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }

    /* Create a data handle for the second level message text. */
    rc = cwbDB_CreateDataHandle(&secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }


    rc = SMP_OK;

    /* Prompt user for request types until user quits. */
    while (rc != SMP_QUIT) {
        printf("\nChoose (by number) one type of requests to perform,\n");
        printf("\tor type Q to quit:\n");
        printf("\t\t1. Catalog\n");
        printf("\t\t2. Native Database\n");
        printf("\t\t3. SQL\n");
        printf(">> ");
        fflush(stdout);

        /* Get input from user. */
        gets(input);

        switch (input[0]) {
        case '1':
           /* Call function CatalogRequest() to perform a catalog request. */
           CatalogRequest();
           break;
        case '2':
           /* Call function NDBRequest() to perform a native database request. */
           NDBRequest();
           break;
        case '3':
           /* Call function SQLRequest() to perform a SQL request. */
           SQLRequest();
           break;
        case 'q':
        case 'Q':
           /* User types in q or Q to quit. */
           rc = SMP_QUIT;
           break;
        default:
           printf("\nInvalid request type, please try again.\n");
          break;
        } /* endswitch */

    } /* endwhile */

    /* Delete data handles used for ReturnHostErrorInfo. */
    cwbDB_DeleteDataHandle(hostMsgID, msgHandle);
    cwbDB_DeleteDataHandle(firstLevelMessage, msgHandle);
    cwbDB_DeleteDataHandle(secondLevelMessage, msgHandle);

    /* Stop communication with AS/400 system. */
    printf("\nStopping server . . .");
    rc = cwbDB_StopServer(connectionHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_StopServer failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }

    /* Delete connection handle. */
    printf("\nDeleting connection handle . . .");
    rc = cwbDB_DeleteConnectionHandle(connectionHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_DeleteConnectionHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return(SMP_ERR);
    }

    /* Delete error handle. */
    cwbSV_DeleteErrHandle(msgHandle);

    return(SMP_OK);

} /* end of main */


/*******************************************************************
*
* Function:  CatalogRequest
*
* Purpose:   Display a list of catalog types and prompt user to choose which type
*              to retrieve.
*
* Parameters:  none
*
********************************************************************/
void CatalogRequest(void)
{
    int         rc = 0;
    char        input[BUFLEN];
    cwbSV_ErrHandle msgHandle;
    cwbDB_RequestHandle      request;

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Create catalog request handle for subsequest API calls that request */
    /*    catalog information. */
    rc=cwbDB_CreateCatalogRequestHandle(connectionHandle, &request, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateCatalogRequestHandle failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user to choose which type of catalog information to retrieve. */
    printf("\nChoose (by number) one type of catalog information to retrieve,\n");
    printf("\tor type Q to quit:\n");
    printf("\t\t1. Library\n");
    printf("\t\t2. SQL Package\n");
    printf("\t\t3. File\n");
    printf(">> ");
    fflush(stdout);

    /* Get input from user. */
    gets(input);

    switch (input[0]) {
    case '1':
       /* Call function RetrieveLibrary() to retrieve library informations. */
       RetrieveLibrary(request);
       break;
    case '2':
       /* Call function RetrieveSQLPackage() to retrieve package informations. */
       RetrieveSQLPackage(request);
       break;
    case '3':
       /* Call function RetrieveFile() to retrieve file informations. */
       RetrieveFile(request);
       break;
    case 'q':
    case 'Q':
       /* User chooses to quit. */
       break;
    default:
       printf("\nInvalid catalog type, please try again.\n");
      break;
    } /* endswitch */

    /* Delete catalog request handle. */
    if ((rc=cwbDB_DeleteCatalogRequestHandle(request, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_DeleteCatalogRequestHandle failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Delete error handle. */
    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of CatalogRequest */


/*******************************************************************
*
* Function:  RetrieveLibrary
*
* Purpose:   Interactively prompt user for parameters needed to retrieve library
*             informations.  Call function ReturnData() to return result
*             to the client and display the result on the screen as well
*             as save the result to a file.
*
* Parameter:
*    cwbDB_RequestHandle  requestHandle - input
*
*******************************************************************/
void RetrieveLibrary(cwbDB_RequestHandle requestHandle)
{
    int         rc = 0;
    char        input[BUFLEN];
    cwbSV_ErrHandle msgHandle;
    char    buffer[BUFLEN], libname[LIBLEN], *libptr;
    unsigned long       information =0;

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Prompt user for name of libraries to obtain information about. */
    /* If not provided, the default of *USRLIBL will be used. */
    printf("\nEnter a character string with or without a wildcard that ");
    printf("specifies the libraries to obtain information about.  ");
    printf("Or just hit ENTER to use the default of *USRLIBL.\n");
    printf(">> ");
    fflush(stdout);
    fgets(buffer, BUFLEN, stdin);
    *(buffer+strlen(buffer)-1) = '\0';
    libptr = buffer;

    /* If user specifies a library string, set the library name for the request  */
    /* to the string.  If not specified, the default of *USRLIBL will be used. */
    if (strlen(buffer)>1) {
        strtoupper(libptr);
        strncpy(libname, libptr, LIBLEN);
        rc = cwbDB_SetLibraryName(requestHandle, libname, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_SetLibraryName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    }

    /* Ask user what information is to be retrieved for the libraries. */
    printf("\nSelect the library information to be retrieved.\n");
    printf("\t\t1. NAME\n");
    printf("\t\t2. NAME and OWNER\n");
    printf("\t\t3. NAME and DESCRIPTION and OWNER\n");
    printf(">> ");
    fflush(stdout);

    /* Get input from user. */
    gets(input);

    /* Set retrieve information parameter according to user selection. */
    switch (input[0]) {
    case '1':
       /* Retrieve library name. */
       information = CWBDB_GET_LIBRARY_NAME;
       break;
    case '2':
       /* Retrieve library name and owner. */
       information = (CWBDB_GET_LIBRARY_NAME | CWBDB_GET_LIBRARY_OWNER);
       break;
    case '3':
       /* Retrieve library name and description and owner. */
       information = (CWBDB_GET_LIBRARY_NAME | CWBDB_GET_LIBRARY_DESC
                       | CWBDB_GET_LIBRARY_OWNER);
       break;
    default:
       printf("\nInvalid choice of library information.  Using API default value.\n");
       break;
    } /* endswitch */

    /* Call API to perform retrieval. */
    printf("\nRetrieving library information . . .");

    /* Instruct the AS/400 server to return the error or diagnostic information. */
    rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                       hostMsgID, firstLevelMessage,
                                        secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    rc = cwbDB_RetrieveLibraryInformation(requestHandle,
                                               information,
                                               msgHandle);
    DisplayHostError(requestHandle);

    if (rc != CWB_OK) {
        printf("\ncwbDB_RetrieveLibraryInformation failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    } else {
        ReturnData(requestHandle);
    } /* endif */

    /* Delete error handle. */
    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of RetrieveLibrary */


/*******************************************************************
*
* Function:  RetrieveSQLPackage
*
* Purpose:   Interactively prompt user for parameters needed to retrieve SQL package
*             informations.  Call function ReturnData() to return result
*             to the client and display the result on the screen as well
*             as save the result to a file.
*
* Parameter:
*    cwbDB_RequestHandle  requestHandle - input
*
*******************************************************************/
void RetrieveSQLPackage(cwbDB_RequestHandle requestHandle)
{
    int         rc = 0;
    char        input[BUFLEN];
    cwbSV_ErrHandle msgHandle;
    char    library[BUFLEN], libname[LIBLEN], *libptr;
    char    package[BUFLEN], pkgname[PKGLEN], *pkgptr;
    unsigned long       information=0;

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Prompt user for name of libraries to obtain package information from. */
    /* If not provided, the default of *USRLIBL will be used. */
    printf("\nEnter a character string with or without a wildcard that ");
    printf("specifies the libraries to obtain SQL package information from.  ");
    printf("Or just hit ENTER to use the default of *USRLIBL.\n");
    printf(">> ");
    fflush(stdout);
    fgets(library, BUFLEN, stdin);
    *(library+strlen(library)-1) = '\0';
    libptr = library;

    /* If user specifies a library string, set the library name for the request  */
    /* to the string.  If not specified, the default of *USRLIBL will be used. */
    if (strlen(library)>1) {
        strtoupper(libptr);
        strncpy(libname, libptr, LIBLEN);
        rc = cwbDB_SetLibraryName(requestHandle, libname, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_SetLibraryName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    }

    /* Prompt user for name of packages to obtain information about. */
    /* If not provided, the default of *ALL will be used. */
    printf("\nEnter a character string with or without a wildcard that ");
    printf("specifies the SQL packages to obtain information about.  ");
    printf("Or just hit ENTER to use the default of *ALL.\n");
    printf(">> ");
    fflush(stdout);
    fgets(package, BUFLEN, stdin);
    *(package+strlen(package)-1) = '\0';
    pkgptr = package;

    /* If user specifies a library string, set the library name for the request  */
    /* to the string.  If not specified, the default of *USRLIBL will be used. */
    if (strlen(package)>1) {
        strtoupper(pkgptr);
        strncpy(pkgname, pkgptr, PKGLEN);
        rc = cwbDB_SetPackageName(requestHandle, pkgname, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_SetPackageName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    }

    /* Ask user what information is to be retrieved for the SQL packages. */
    printf("\nSelect the SQL package information to be retrieved.\n");
    printf("\t\t1. SQL PACKAGE NAME\n");
    printf("\t\t2. SQL PACKAGE NAME and DESCRIPTION\n");
    printf("\t\t3. SQL PACKAGE LIBRARY and NAME and DESCRIPTION\n");
    printf(">> ");
    fflush(stdout);

    /* Get input from user. */
    gets(input);

    /* Set retrieve information parameter according to user selection. */
    switch (input[0]) {
    case '1':
       /* Retrieve SQL package name. */
       information = CWBDB_GET_SQLPKG_NAME;
       break;
    case '2':
       /* Retrieve SQL package name and description. */
       information = (CWBDB_GET_SQLPKG_NAME | CWBDB_GET_SQLPKG_DESC);
       break;
    case '3':
       /* Retrieve SQL package library and name and description. */
       information = (CWBDB_GET_SQLPKG_LIB | CWBDB_GET_SQLPKG_NAME
                        | CWBDB_GET_SQLPKG_DESC);
       break;
    default:
       printf("\nInvalid choice of SQL package information.  Using API default value.\n");
       break;
    } /* endswitch */

    /* Call API to perform retrieval. */
    printf("\nRetrieving SQL package information . . .");

    /* Instruct the AS/400 server to return the error or diagnostic information. */
    rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                       hostMsgID, firstLevelMessage,
                                        secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    rc = cwbDB_RetrieveSQLPackageInformation(requestHandle,
                                                   information,
                                                   msgHandle);
    DisplayHostError(requestHandle);

    if (rc != CWB_OK) {
        printf("\ncwbDB_RetrieveSQLPackageInformation failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    } else {
        /* Call function ReturnData() to return result to client, display result */
        /* on the screen and save result to a file. */
        ReturnData(requestHandle);
    } /* endif */

    /* Delete error handle. */
    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of RetrieveSQLPackage */


/*******************************************************************
*
* Function:  RetrieveFile
*
* Purpose:   Interactively prompt user for parameters needed to retrieve file
*             informations.  Call function ReturnData() to return result
*             to the client and display the result on the screen as well
*             as save the result to a file.
*
* Parameter:
*    cwbDB_RequestHandle  requestHandle - input
*
*******************************************************************/
void RetrieveFile(cwbDB_RequestHandle requestHandle)
{
    int         rc = 0;
    char        input[BUFLEN];
    cwbSV_ErrHandle msgHandle;
    char    lib_buffer[BUFLEN], libname[LIBLEN], *libptr;
    char    file_buffer[BUFLEN], filename[FILELEN], *fileptr;
    unsigned long       information =0;

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Prompt user for name of libraries to obtain file information from. */
    /* If not provided, the default of *USRLIBL will be used. */
    printf("\nEnter a character string with or without a wildcard that ");
    printf("specifies the libraries to obtain file information from.  ");
    printf("Or just hit ENTER to use the default of *USRLIBL.\n");
    printf(">> ");
    fflush(stdout);
    fgets(lib_buffer, BUFLEN, stdin);
    *(lib_buffer+strlen(lib_buffer)-1) = '\0';
    libptr = lib_buffer;

    /* If user specifies a library string, set the library name for the request  */
    /* to the string.  If not specified, the default of *USRLIBL will be used. */
    if (strlen(lib_buffer)>1) {
        strtoupper(libptr);
        strncpy(libname, libptr, LIBLEN);
        rc = cwbDB_SetLibraryName(requestHandle, libname, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_SetLibraryName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    }

    /* Prompt user for name of files to obtain information about. */
    /* If not provided, the default of *ALL will be used. */
    printf("\nEnter a character string with or without a wildcard that ");
    printf("specifies the files to obtain information about.  ");
    printf("Or just hit ENTER to use the default of *ALL.\n");
    printf(">> ");
    fflush(stdout);
    fgets(file_buffer, BUFLEN, stdin);
    *(file_buffer+strlen(file_buffer)-1) = '\0';
    fileptr = file_buffer;

    /* If user specifies a library string, set the library name for the request  */
    /* to the string.  If not specified, the default of *USRLIBL will be used. */
    if (strlen(file_buffer)>1) {
        strtoupper(fileptr);
        strncpy(filename, fileptr, FILELEN);
        rc = cwbDB_SetFileName(requestHandle, filename, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_SetFileName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    }

    /* Ask user what information is to be retrieved for the files. */
    printf("\nSelect the file information to be retrieved.\n");
    printf("\t\t1. LIBRARY NAME and FILE NAME\n");
    printf("\t\t2. FILE NAME and ATTRIBUTE and AUTHORITY\n");
    printf("\t\t3. FILE NAME and DESCRIPTION and COLUMN COUNTS\n");
    printf(">> ");
    fflush(stdout);

    /* Get input from user. */
    gets(input);

    /* Set retrieve information parameter according to user selection. */
    switch (input[0]) {
    case '1':
       /* Retrieve library name and file name. */
       information = (CWBDB_GET_FILE_LIB | CWBDB_GET_FILE_NAME);
       break;
    case '2':
       /* Retrieve file name and attribute and authority. */
       information = (CWBDB_GET_FILE_NAME | CWBDB_GET_FILE_ATTRIB
                        | CWBDB_GET_FILE_AUTH);
       break;
    case '3':
       /* Retrieve file name and description and column counts. */
       information = (CWBDB_GET_FILE_NAME | CWBDB_GET_FILE_DESC
                        | CWBDB_GET_FILE_COL_CNT);
       break;
    default:
       printf("\nInvalid choice of file information.  Using API default value.\n");
       break;
    } /* endswitch */

    /* Call API to perform retrieval. */
    printf("\nRetrieving file information . . .");

    /* Instruct the AS/400 server to return the error or diagnostic information. */
    rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                       hostMsgID, firstLevelMessage,
                                        secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    rc = cwbDB_RetrieveFileInformation(requestHandle,
                                            information,
                                            msgHandle);
    DisplayHostError(requestHandle);

    if (rc != CWB_OK) {
        printf("\ncwbDB_RetrieveFileInformation failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    } else {
        /* Call function ReturnData() to return result to client, display result */
        /* on the screen and save result to a file. */
        ReturnData(requestHandle);
    } /* endif */

    /* Delete error handle. */
    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of RetrieveFile */


/*******************************************************************
*
*  Function:  ReturnData
*
*  Purpose:   Return the data from the server to the client.  The requested data
*              is the result from a Catalog request or a SQL request.
*              Both packed and zoned data types are converted to double.
*              DisplayData() is called to format the data, display the result
*              on the screen and save the result to an output file.
*
*  Parameters:
*       cwbDB_RequestHandle  request - input, a catalog or SQL request
*                                            that contains result data
*
*******************************************************************/
void ReturnData(cwbDB_RequestHandle requestHandle)
{
    int         i;
    int         rc = 0;
    unsigned long       columnCount;          /* Number of columns in the result. */
    unsigned long       location;               /* Either system or local. */
    unsigned long       columnPosition;
    signed short        dataType;
    unsigned long       columnLength = 0;
    unsigned short      columnPrecision = 0;
    unsigned short      columnScale = 0;
    cwbSV_ErrHandle msgHandle;          /* Error handle. */
    cwbDB_DataHandle  data;             /* Data handle for actual data. */
    cwbDB_DataHandle  indicator;        /* Data handle for indicators. */
    cwbDB_FormatHandle   format;        /* Format handle for data format. */

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Create data handle for actual data. */
    if ((rc = cwbDB_CreateDataHandle(&data, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Create data handle for indicators. */
    if ((rc = cwbDB_CreateDataHandle(&indicator, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Create data format handle for the description of the result data. */
    if ((rc = cwbDB_CreateDataFormatHandle(connectionHandle, &format, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_CreateDataFormatHandle failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Request that the data format in the result set be returned. */
    rc = cwbDB_ReturnDataFormat(requestHandle, format, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnDataFormat failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the requested data format. */
    printf("\nGetting the data format . . .");
    if ((rc = cwbDB_GetData(requestHandle, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_GetData failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the number of column in the result set. */
    rc = cwbDB_GetColumnCount(format, &columnCount, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetColumnCount failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Check to see if any column is either a packed or zoned data type. */
    /* Convert packed and zoned data types to double. */
    printf("\nConverting any packed and zoned data types to double . . .");
    location = CWBDB_LOCAL;             /* Get the local client format. */
    for (i=1; i <= (int)columnCount; i++) {
        /* Get the data type of each column. */
        columnPosition = i;
        rc = cwbDB_GetColumnType(format, location, columnPosition,
                                       &dataType, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_GetColumnType failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        } else {
            /* Set packed or zoned columns to double. */
            /* Column length, precision, and scale are not used for double. */
            if ((dataType == CWBDB_PCPACKED) || (dataType == CWBDB_PCZONED)) {
                dataType = CWBDB_PCDOUBLE;
                rc = cwbDB_SetClientColumnToNumeric(format, columnPosition,
                                                           dataType, columnLength,
                                                           columnPrecision,
                                                           columnScale, msgHandle);
                if (rc != CWB_OK) {
                    printf("\ncwbDB_SetClientColumnToNumeric failed ");
                    printf("with return code %d.\n", rc);
                    resolveErr(msgHandle);
                    return;
                }
            } else {
                /* All other data types are returned in the default format. */
            } /* endif */
        } /* endif */
    } /* endfor */


    /* Request that the data and indicator in the result set be returned */
    /* to the client in the converted data format. */
    rc = cwbDB_ReturnData(requestHandle, data, indicator, format, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnData failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the requested data. */
    printf("\nGetting the requested data to the client . . .");
    if ((rc = cwbDB_GetData(requestHandle, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_GetData failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Call function DisplayData() to parse and display the data. */
    DisplayData(data, indicator, format, columnCount);

    /* Delete data handles, format handle and error handle. */
    cwbDB_DeleteDataHandle(data, msgHandle);
    cwbDB_DeleteDataHandle(indicator, msgHandle);
    cwbDB_DeleteDataFormatHandle(format, msgHandle);
    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of ReturnData */


/*******************************************************************
*
*  Function:  DisplayData
*
*  Purpose:   Get column informations:  column name, length of column name,
*               data type, data length.  Set column display width to the greater of
*               name length or data length.  Call PrintData() to display result on
*               screen and save to an output file.
*
*  Parameters:
*       cwbDB_DataHandle data  - input, data handle containing result data
*       cwbDB_DataHandle indicator  - input, data handle containing null indicators
*       cwbDB_FormatHandle format   - input, format handle containing data format
*       unsigned long   columnCount   - input, number of columns in result set
*
********************************************************************/
void DisplayData(cwbDB_DataHandle data, cwbDB_DataHandle indicator,
                   cwbDB_FormatHandle format, unsigned long  columnCount)
{
    unsigned int i;
    unsigned int j;
    unsigned int rc = 0;
    cwbSV_ErrHandle msgHandle;
    unsigned long       location = CWBDB_LOCAL;  /* Work with local format. */
    unsigned long       rowSize;        /* Size of each row. */
    char *              dataBuffer;     /* Pointer to the actual data buffer. */
    char *              indicatorBuffer; /* Pointer to the indicator buffer. */
    unsigned long       dataLength;     /* Length of the data buffer. */
    unsigned long       indicatorLength; /* Length of the indicator buffer. */
    unsigned long       rowCount;       /* Number of rows. */
    unsigned long       columnPosition; /* Position of column. */
    char *              defaultName="N/A";      /* Default column name when */
                                                    /*  one is not contained in the result. */
    struct column *     columnInfo;  /* Array of column structures. */
    unsigned long     totalWidth=0;  /* Total display width of all the columns plus spacings. */
    cwbDB_DataHandle  *nameHandle;    /* Array of data handles for column names. */

    /* Allocate memory for an array of data handles, one for each column name. */
    nameHandle = (cwbDB_DataHandle *) malloc (sizeof(cwbDB_DataHandle) * columnCount);

    /* Allocate memory for an array of column structures, one for each column. */
    columnInfo = (struct column *)malloc(sizeof(struct column) * columnCount);


    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Get the length of the indicator buffer. */
    rc = cwbDB_GetDataLength(indicator, &indicatorLength, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataLength failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* If the indicator buffer has positive length, get the pointer to the  */
    /* indicator buffer. */
    if (dataLength > 0) {
        rc = cwbDB_GetDataPointer(indicator, &indicatorBuffer, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_GetDataPointer failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    } else {
        printf("\nThe indicator buffer is empty.\n");
        return;
    } /* endif */

    /* Get the length of the actual data buffer. */
    rc = cwbDB_GetDataLength(data, &dataLength, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataLength failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* If the data buffer has positive length, get the pointer to the  */
    /* actual data buffer. */
    if (dataLength > 0) {
        rc = cwbDB_GetDataPointer(data, &dataBuffer, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_GetDataPointer failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    } else {
        printf("\nThe data buffer is empty.\n");
        return;
    } /* endif */

    /* Get the size of each row in the result set. */
    rc = cwbDB_GetRowSize(format, location, &rowSize, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetRowSize failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* The number of rows is data length divided by row size. */
    rowCount = dataLength / rowSize;

    /* Create a data handle for each column names. */
    for (j=0; j<columnCount; j++) {
        rc = cwbDB_CreateDataHandle(&nameHandle[j], msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_CreateDataHandle failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    } /* endfor */

    /* For each column, get its name, the length of its name, */
    /* its data type, and its data length.  The width of the column in the */
    /* display table is set to the greater of the length of the name or the */
    /* display length of the data.   The displayed data length is set according */
    /* to the column's data type. */
    /* Store the informations in the column structure columnInfo. */

    printf("\nGetting column informations . . .");

    for (i=0 ; i < columnCount ; i++ ) {
         columnPosition = i + 1;        /* Column positions start at 1. */

         /* Get column name into the data handle nameHandle. */
         rc = cwbDB_GetColumnName(format, columnPosition,
                                         nameHandle[i], msgHandle);
         if (rc != CWB_OK) {
            printf("\ncwbDB_GetColumnName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
         } else {
             /* Get length of name handle and store it in the nameLength field of */
             /* columnInfo[i]. */
             rc = cwbDB_GetDataLength(nameHandle[i], &columnInfo[i].nameLength,
                                            msgHandle);
             if (rc != CWB_OK) {
                 printf("\ncwbDB_GetDataLength failed with return code %d.\n", rc);
                 resolveErr(msgHandle);
                 return;
             } else {
                 /* If name length is greater than 0, get column name into */
                 /* the name field of columnInfo[i]. */
                 if (columnInfo[i].nameLength > 0) {
                     rc = cwbDB_GetDataPointer(nameHandle[i],
                                                    &columnInfo[i].name,
                                                    msgHandle);
                     if (rc != CWB_OK) {
                         printf("\ncwbDB_GetDataPointer failed with ");
                         printf("return code %d.\n", rc);
                         resolveErr(msgHandle);
                         return;
                     }
                 } else {
                     /* Column name buffer is empty.  */
                     /* Use default string 'N/A' for column name. */
                     strcpy(columnInfo[i].name, defaultName);
                     /* Set length of column name plus null terminator. */
                     columnInfo[i].nameLength = strlen(columnInfo[i].name)+1;
                 } /* endif */
             } /* endif */
         } /* endif */

         /* Get column data length and store it in the columnLength field of columnInfo[i]. */
         rc = cwbDB_GetColumnLength(format, location, columnPosition,
                                        &columnInfo[i].columnLength, msgHandle);
         if (rc != CWB_OK) {
             printf("\ncwbDB_GetColumnLength failed with return code %d.\n", rc);
             resolveErr(msgHandle);
             return;
         }

         /* Get column data type and store it in the type field of columnInfo[i]. */
         rc = cwbDB_GetColumnType(format, location, columnPosition,
                                        &columnInfo[i].type, msgHandle);
         if (rc != CWB_OK) {
             printf("\ncwbDB_GetColumnType failed with return code %d.\n", rc);
             resolveErr(msgHandle);
             return;
         } else {
             /* Check the data type and determine the space needed for displaying */
             /* the data.  This space is compared to the length of the column name */
             /* which is used as column heading, and the greater of the two */
             /* values is stored in the width field of columnInfo[i]. */

             switch (columnInfo[i].type) {
             case CWBDB_PCSHORT:
                /* Short integer requires display length of 6, including the sign. */
                columnInfo[i].width = 6;
                if (columnInfo[i].nameLength > columnInfo[i].width) {
                    columnInfo[i].width = columnInfo[i].nameLength;
                }
                break;
             case CWBDB_PCLONG:
                /* Long integer requires display length of 11, including the sign. */
                columnInfo[i].width = 11;
                if (columnInfo[i].nameLength > columnInfo[i].width) {
                    columnInfo[i].width = columnInfo[i].nameLength;
                }
                break;
             case CWBDB_PCFLOAT:
                /* Float requires display length of 16, including sign and exponent. */
                columnInfo[i].width = 16;
                if (columnInfo[i].nameLength > columnInfo[i].width) {
                    columnInfo[i].width = columnInfo[i].nameLength;
                }
                break;
             case CWBDB_PCDOUBLE:
                /* Double requires display length of 16, including sign and exponent. */
                columnInfo[i].width = 16;
                if (columnInfo[i].nameLength > columnInfo[i].width) {
                    columnInfo[i].width = columnInfo[i].nameLength;
                }
                break;
             case CWBDB_PCSTRING:
             case CWBDB_PCVARSTRING:
             case CWBDB_PCGRAPHIC:
             case CWBDB_PCVARGRAPHIC:
                /* The display width of all string data types is the same as their */
                /*  column data length. */
                columnInfo[i].width = columnInfo[i].columnLength;
                if (columnInfo[i].nameLength > columnInfo[i].width) {
                    columnInfo[i].width = columnInfo[i].nameLength;
                }
                break;
             default:
                /* Since packed and zoned should have been converted to double, */
                /* any other PC data types would be invalid. */
                printf("\nColumn position %d is of invaid data type %d.\n",
                                    columnPosition, columnInfo[i].type);
                /* Use the given column data length. */
                columnInfo[i].width = columnInfo[i].columnLength;
                if (columnInfo[i].nameLength > columnInfo[i].width) {
                    columnInfo[i].width = columnInfo[i].nameLength;
                }
               break;
             } /* endswitch */

             /* Add column width to the total value. */
             totalWidth += columnInfo[i].width;

         } /* endif */
    } /* endfor */

    /* Add spacings to the total width. */
    totalWidth += 2 * (columnCount-1);

    /* Call function PrintData() to display result on the screen and */
    /*  save to an output file. */
    PrintData(totalWidth, columnCount, rowCount, columnInfo, dataBuffer, rowSize,
               indicatorBuffer);

    /* Delete data handles, and error handle. */
    for (j=0; j<columnCount ; j++) {
    cwbDB_DeleteDataHandle(nameHandle[j], msgHandle);
    } /* endfor */

    cwbSV_DeleteErrHandle(msgHandle);

    free(columnInfo);

    return;
} /* end of DisplayData */


/*******************************************************************
*
*  Function:  PrintData
*
*  Purpose:   Display formatted data to the screen and save to an output file.
*
*  Parameters:
*       unsigned long   totalWidth  - input, total display width of all columns
*       unsigned long   columnCount - input, number of columns
*       unsigned long   rowCount - input, number of rows
*       struct column   columnInfo[] - input, array of column structures
*       char *  dataBuffer      - input, pointer to the actual data buffer
*       unsigned long   rowSize  - input, size of a row of data in the result set
*       char *  indicatorBuffer   - input, pointer to the indicator buffer
*
********************************************************************/
void PrintData(unsigned long  totalWidth, unsigned long  columnCount,
                unsigned long  rowCount, struct column  columnInfo[],
                char *  dataBuffer, unsigned long  rowSize,
                char *  indicatorBuffer)
{
    unsigned int        j;
    unsigned long       rowIndex;
    unsigned long       colIndex;
    char    buffer[BUFSIZ],  *outFile;  /* Output file. */
    FILE  * fp;                         /* File pointer. */
    char  *    prtBuffer;        /* Print buffer for each row of output. */
    int         bufferCount;            /* Position counter for the print buffer. */
    unsigned long   bufferIndex;    /* Indicate position in data buffer. */
    unsigned long   indicatorIndex;  /* Index position in indicator buffer. */
    signed short *      indicatorPtr;  /* For temporary storage of indicator. */
    short *           shortBuffer;    /* For temporary storage of short data. */
    long *          longBuffer;     /* For temporary storage of long data. */
    float *         floatBuffer;    /* For temporary storage of float data. */
    double *        doubleBuffer;   /* For temporary storage of double data. */

    prtBuffer = (char *) malloc(sizeof(char*) * totalWidth);

    /* Prompt user for the name of the output file to save result data to. */
    printf("\nEnter output file name, could include path: ");
    fflush(stdout);
    fgets(buffer, BUFSIZ, stdin);
    *(buffer+strlen(buffer)-1) = '\0';
    outFile = buffer;

    if ((fp = fopen(outFile, "w")) == NULL) {
        printf("\nError opening output file: %s\n", outFile);
        return;
    } else {
        /* Print the column headings. */
        memset(prtBuffer, ' ', totalWidth);      /* Initialize print buffer to blanks. */
        bufferCount=0;
        for (j=0; j < columnCount; j++) {
            memcpy(prtBuffer+bufferCount, columnInfo[j].name, columnInfo[j].nameLength);
            bufferCount += (columnInfo[j].width + 2);
        }
        prtBuffer[bufferCount-2] = '\0';
        fprintf(fp, "%s\n", prtBuffer);
        printf("%s\n", prtBuffer);

        /* For each row of data, print the columns  */
        /* according to the format information in columnInfo[]. */
        /* Right justify the numeric data. */
        /* Add two spaces between each column for readability. */

        for (rowIndex=0; rowIndex<rowCount; rowIndex++) {
            bufferIndex = (rowIndex * rowSize);
            memset(prtBuffer, ' ', totalWidth);
            bufferCount=0;
            indicatorIndex = rowIndex * columnCount * 2;

            for (colIndex=0; colIndex<columnCount; colIndex++) {
                /* Check indicator buffer to see if column is NULL. */
                indicatorPtr = (signed short *) (indicatorBuffer + indicatorIndex);

                if (*(indicatorPtr) == -1) {
                    /* Value is NULL.  Do not print. */
                } else {
                    switch (columnInfo[colIndex].type) {
                    case CWBDB_PCSHORT:
                       shortBuffer = (short *) (dataBuffer + bufferIndex);
                       sprintf(prtBuffer+bufferCount+columnInfo[colIndex].width-6,
                               "%6d", *shortBuffer);
                       prtBuffer[strlen(prtBuffer)] = ' '; /* Print space over null terminator. */
                       break;
                    case CWBDB_PCLONG:
                       longBuffer = (long *) (dataBuffer + bufferIndex);
                       sprintf(prtBuffer+bufferCount+columnInfo[colIndex].width-11,
                               "%11ld", *longBuffer);
                       prtBuffer[strlen(prtBuffer)] = ' '; /* Print space over null terminator. */
                       break;
                    case CWBDB_PCFLOAT:
                       floatBuffer = (float *) (dataBuffer + bufferIndex);
                       sprintf(prtBuffer+bufferCount+columnInfo[colIndex].width-16,
                               "%.8E", *floatBuffer);
                       prtBuffer[strlen(prtBuffer)] = ' '; /* Print space over null terminator. */
                       break;
                    case CWBDB_PCDOUBLE:
                       doubleBuffer = (double *) (dataBuffer + bufferIndex);
                       sprintf(prtBuffer+bufferCount+columnInfo[colIndex].width-16,
                               "%.8E", *doubleBuffer);
                       prtBuffer[strlen(prtBuffer)] = ' '; /* Print space over null terminator. */
                       break;
                    case CWBDB_PCSTRING:
                    case CWBDB_PCVARSTRING:
                    case CWBDB_PCGRAPHIC:
                    case CWBDB_PCVARGRAPHIC:
                       memcpy(prtBuffer+bufferCount, (dataBuffer + bufferIndex),
                                 columnInfo[colIndex].columnLength);
                       break;
                    default:
                      break;
                    } /* endswitch */
                } /* endif */

                indicatorIndex += 2;
                bufferIndex += columnInfo[colIndex].columnLength;
                bufferCount += (columnInfo[colIndex].width + 2);

            } /* endfor */

            prtBuffer[bufferCount-2] = '\0';
            fprintf(fp, "%s\n", prtBuffer);
            printf("%s\n", prtBuffer);

        } /* endfor */
    } /* endif */

    /* Close output file. */
    fclose(fp);

    free(prtBuffer);

    return;
} /* end PrintData */


/******************************************************************
*
*  Function:  NDBRequest
*
*  Purpose:   Allow user to perform operations on AS/400 file objects.  User can
*              choose to create a source physical file, create a duplicate file
*              from a base file with the options to copy or not copy the data,
*              or delete a file.  User is prompted interactively for required
*              informations.
*
*  Parameter:  none
*
*******************************************************************/
void NDBRequest(void)
{
    int         rc = 0;
    char        input[BUFLEN];
    cwbSV_ErrHandle msgHandle;
    cwbDB_RequestHandle      requestHandle;
    char    lib_buffer[BUFLEN], libname[LIBLEN], *libptr;
    char    file_buffer[BUFLEN], filename[FILELEN], *fileptr;
    char    baseLib_buffer[BUFLEN], baseLibname[LIBLEN], *baseLibptr;
    char    baseFile_buffer[BUFLEN], baseFilename[FILELEN], *baseFileptr;
    cwb_Boolean         copyDataIndicator;

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Create NDB request handle for subsequest API calls that request */
    /* operations to be performed with AS/400 file objects. */
    rc=cwbDB_CreateNDBRequestHandle(connectionHandle, &requestHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateNDBRequestHandle failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user for name of library to locate the file. */
    /* If not provided, the default of *CURLIB will be used. */
    printf("\nEnter the name of the library to locate the file.  ");
    printf("Or just hit ENTER to use the default of *CURLIB.\n");
    printf(">> ");
    fflush(stdout);
    fgets(lib_buffer, BUFLEN, stdin);
    *(lib_buffer+strlen(lib_buffer)-1) = '\0';
    libptr = lib_buffer;

    /* If user specifies a library string, set the library name for the request  */
    /* to the string.  If not specified, the default of *CURLIB will be used. */
    if (strlen(lib_buffer)>1) {
        strtoupper(libptr);
        strncpy(libname, libptr, LIBLEN);
        rc = cwbDB_SetLibraryName(requestHandle, libname, msgHandle);
        if (rc != CWB_OK) {
            printf("\ncwbDB_SetLibraryName failed with return code %d.\n", rc);
            resolveErr(msgHandle);
            return;
        }
    }

    /* Prompt user for name of the file to operate on.  */
    printf("\nEnter the name of the file: ");
    fflush(stdout);
    fgets(file_buffer, BUFLEN, stdin);
    *(file_buffer+strlen(file_buffer)-1) = '\0';
    fileptr = file_buffer;

    /* Set the file name for the request to the input string. */
    strtoupper(fileptr);
    strncpy(filename, fileptr, FILELEN);
    rc = cwbDB_SetFileName(requestHandle, filename, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_SetFileName failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user to choose which type of NDB operations to perform. */
    printf("\nChoose (by number) one type of NDB operations to perform,\n");
    printf("\tor type Q to quit:\n");
    printf("\t\t1. Create a source physical file\n");
    printf("\t\t2. Duplicate from a base file\n");
    printf("\t\t3. Delete a file\n");
    printf(">> ");
    fflush(stdout);

    /* Get input from user. */
    gets(input);

    switch (input[0]) {
    case '1':
       /* Create a source physical file. */

       /* Instruct the AS/400 server to return the error or diagnostic information. */
       rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                          hostMsgID, firstLevelMessage,
                                           secondLevelMessage, msgHandle);
       if (rc != CWB_OK) {
           printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }

       rc = cwbDB_CreateSourcePhysicalFile(requestHandle, msgHandle);
       DisplayHostError(requestHandle);

       if (rc != CWB_OK) {
           printf("\ncwbDB_CreateSourcePhysicalFile failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }
       break;
    case '2':
       /* Create a duplicate file from the base file. */

       /* Prompt user for name of library to locate the base file. */
       /* If not provided, the default of *CURLIB will be used. */
       printf("\nEnter the name of the library to locate the base file.  ");
       printf("Or just hit ENTER to use the default of *CURLIB.\n");
       printf(">> ");
       fflush(stdout);
       fgets(baseLib_buffer, BUFLEN, stdin);
       *(baseLib_buffer+strlen(baseLib_buffer)-1) = '\0';
       baseLibptr = baseLib_buffer;
       strtoupper(baseLibptr);
       strncpy(baseLibname, baseLibptr, LIBLEN);

       /* Prompt user for the name of the base file. */
       printf("\nEnter the name of the base file: ");
       fflush(stdout);
       fgets(baseFile_buffer, BUFLEN, stdin);
       *(baseFile_buffer+strlen(baseFile_buffer)-1) = '\0';
       baseFileptr = baseFile_buffer;
       strtoupper(baseFileptr);
       strncpy(baseFilename, baseFileptr, FILELEN);

       /* Set the base library and file name for the request. */
       rc = cwbDB_SetBaseFile(requestHandle, baseLibname,
                                  baseFilename, msgHandle);
       if (rc != CWB_OK) {
           printf("\ncwbDB_SetBaseFile failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }

       /* Ask user if the data from the base file is to be copied into the created file. */
       printf("\nCopy data from the base file?  (Y or N) >> ");
       fflush(stdout);
       gets(input);
       if ((input[0] == 'Y') || (input[0] == 'y')) {
            copyDataIndicator = CWBDB_COPY_DATA;
       } else {
            copyDataIndicator = CWBDB_DO_NOT_COPY_DATA;
       } /* endif */

       /* Instruct the AS/400 server to return the error or diagnostic information. */
       rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                          hostMsgID, firstLevelMessage,
                                           secondLevelMessage, msgHandle);
       if (rc != CWB_OK) {
           printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }

       rc = cwbDB_CreateDuplicateFile(requestHandle, copyDataIndicator, msgHandle);
       DisplayHostError(requestHandle);

       if (rc != CWB_OK) {
           printf("\ncwbDB_CreateDuplicateFile failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }
       break;
    case '3':
       /* Delete a file. */

       /* Instruct the AS/400 server to return the error or diagnostic information. */
       rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                          hostMsgID, firstLevelMessage,
                                           secondLevelMessage, msgHandle);
       if (rc != CWB_OK) {
           printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }

       rc = cwbDB_DeleteFile(requestHandle, msgHandle);
       DisplayHostError(requestHandle);

       if (rc != CWB_OK) {
           printf("\ncwbDB_DeletelFile failed with return code %d.\n", rc);
           resolveErr(msgHandle);
           return;
       }
       break;
    default:
       /* Invalid option for NDB request.  */
       printf("\nInvalid option for NDB request.\n");
      break;
    } /* endswitch */

    return;
} /* end of NDBRequest */


/********************************************************************
*
*  Function:  DisplayHostError
*
*  Purpose:   After the host error information has been requested for a
*               functional API, and the API has been called, this function
*               returns the host error information to the application and
*               displays on the screen.
*
*  Parameter:
*       cwbDB_Request   requestHandle - input, either catalog or NDB request
*
********************************************************************/
void DisplayHostError(cwbDB_RequestHandle  requestHandle)
{
    int         rc;
    cwbSV_ErrHandle msgHandle;
    unsigned long       msgIDLen;       /* Length of host message id. */
    unsigned long       firstMsgLen;    /* Length of first level message. */
    unsigned long       secondMsgLen;   /* Length of second level message. */
    char *      msgIDPtr;    /* Pointer to host message id. */
    char *      firstMsgPtr;    /* Pointer to first level message. */
    char *      secondMsgPtr;   /* Pointer to second level message. */
    char        printBuffer[BUFSIZ];    /* Print buffer. */


    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Get the error information to the client. */
    if ((rc = cwbDB_GetData(requestHandle, msgHandle)) != CWB_OK) {
        printf("\ncwbDB_GetData failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the length of the host message id. */
    rc = cwbDB_GetDataLength(hostMsgID, &msgIDLen, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataLength failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the data pointer for the host message id. */
    rc = cwbDB_GetDataPointer(hostMsgID, &msgIDPtr, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataPointer failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the length of the first level message text. */
    rc = cwbDB_GetDataLength(firstLevelMessage, &firstMsgLen, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataLength failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the data pointer for the first level message text. */
    rc = cwbDB_GetDataPointer(firstLevelMessage, &firstMsgPtr, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataPointer failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the length of the second level message text. */
    rc = cwbDB_GetDataLength(secondLevelMessage, &secondMsgLen, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataLength failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Get the data pointer for the second level message text. */
    rc = cwbDB_GetDataPointer(secondLevelMessage, &secondMsgPtr, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_GetDataPointer failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Display the error information. */
    printf("\nHOST ERROR CLASS:  %d\n", hostErrorClass);
    printf("\nHOST ERROR CODE:  %d\n", hostErrorCode);

    memcpy(printBuffer, msgIDPtr, msgIDLen);
    printBuffer[msgIDLen] = '\0';
    printf("\nHOST MESSAGE ID:  %s\n", printBuffer);

    memcpy(printBuffer, firstMsgPtr, firstMsgLen);
    printBuffer[firstMsgLen] = '\0';
    printf("\nFIRST LEVEL MESSAGE:\n%s\n", printBuffer);

    memcpy(printBuffer, secondMsgPtr, secondMsgLen);
    printBuffer[secondMsgLen] = '\0';
    printf("\nSECOND LEVEL MESSAGE:\n%s\n", printBuffer);

    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of DisplayHostError */


/*******************************************************************
*
*  Function:  SQLRequest
*
*  Purpose:   Allow user to execute a SELECT type SQL statement for read only.
*              Prompt user interactively for informations required for the request.
*              Call function ReturnData() to return result
*              to the client and display the result on the screen as well
*              as save the result to a file.
*
*  Parameter:   none
********************************************************************/
void SQLRequest(void)
{
    int         rc = 0;
    cwbSV_ErrHandle msgHandle;
    cwbDB_RequestHandle      requestHandle;
    char    cur_buffer[BUFLEN], cursor[CURNAMLEN], *curptr;
    char    name_buffer[BUFLEN], sqlName[SQLNAMLEN], *sqlNameptr;
    char    sql_buffer[SQLTXTLEN], sqlTxt[SQLTXTLEN], *sqlTxtptr;
    char    block_buffer[BUFLEN];
    unsigned long       blockCount;
    unsigned char   openOptions = CWBDB_READ; /*Cursor is opened for read only.*/

    /* Create error handle. */
    if ((rc = cwbSV_CreateErrHandle(&msgHandle)) != CWB_OK) {
        printf("\ncwbSV_CreateErrHandle(): Service API failed ");
        printf("with return code %d.\n", rc);
        return;
    }

    /* Create SQL request handle for subsequest API calls that request */
    /*    SQL services. */
    rc=cwbDB_CreateSQLRequestHandle(connectionHandle, &requestHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_CreateSQLRequestHandle failed ");
        printf("with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user for name of cursor. */
    printf("\nEnter a character string for the name of the open cursor: ");
    fflush(stdout);
    fgets(cur_buffer, CURNAMLEN, stdin);
    *(cur_buffer+strlen(cur_buffer)-1) = '\0';
    curptr = cur_buffer;

    /* Set the cursor name for the request.  */
    strtoupper(curptr);
    strncpy(cursor, curptr, CURNAMLEN);
    rc = cwbDB_SetCursorName(requestHandle, cursor, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_SetCursorName failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user for name of statement. */
    printf("\nEnter a character string for the name of the SQL statement: ");
    fflush(stdout);
    fgets(name_buffer, SQLNAMLEN, stdin);
    *(name_buffer+strlen(name_buffer)-1) = '\0';
    sqlNameptr = name_buffer;

    /* Set the statement name for the request.  */
    strtoupper(sqlNameptr);
    strncpy(sqlName, sqlNameptr, SQLNAMLEN);
    rc = cwbDB_SetStatementName(requestHandle, sqlName, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_SetStatementName failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user for the text of a SELECT statement. */
    printf("\nEnter a SELECT statement text: ");
    fflush(stdout);
    fgets(sql_buffer, SQLTXTLEN, stdin);
    *(sql_buffer+strlen(sql_buffer)-1) = '\0';
    sqlTxtptr = sql_buffer;

    /* Set the statement text for the request.  */
    strncpy(sqlTxt, sqlTxtptr, SQLTXTLEN);
    rc = cwbDB_SetStatementText(requestHandle, sqlTxt, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_SetStatementText failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Prompt user for the number of rows to be blocked together when fetching data. */
    printf("\nEnter the block count: ");
    fflush(stdout);
    fgets(block_buffer, BUFLEN, stdin);
    *(block_buffer+strlen(block_buffer)-1) = '\0';
    blockCount = (unsigned long) atol(block_buffer);

    /* Set the block count for the request.  */
    rc = cwbDB_SetBlockCount(requestHandle, blockCount, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_SetBlockCount failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }

    /* Store the request parameters. */
    rc = cwbDB_StoreRequestParameters(requestHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_StoreRequestParameters failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }


    /* Prepare the SQL statement and instruct the AS/400 server to return  */
    /* the error or diagnostic information. */
    rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                       hostMsgID, firstLevelMessage,
                                        secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }
    rc = cwbDB_Prepare(requestHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_Prepare failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }
    DisplayHostError(requestHandle);


    /* Open the cursor for read only and instruct the AS/400 server to return  */
    /* the error or diagnostic information. */
    rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                       hostMsgID, firstLevelMessage,
                                        secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }
    rc = cwbDB_Open(requestHandle, openOptions, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_Open failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }
    DisplayHostError(requestHandle);


    /* Fetch from the open cursor and instruct the AS/400 server to return */
    /* the error or diagnostic information. */
    rc = cwbDB_ReturnHostErrorInfo(requestHandle, &hostErrorClass, &hostErrorCode,
                                       hostMsgID, firstLevelMessage,
                                        secondLevelMessage, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_ReturnHostErrorInfo failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }
    rc = cwbDB_Fetch(requestHandle, msgHandle);
    if (rc != CWB_OK) {
        printf("\ncwbDB_Fetch failed with return code %d.\n", rc);
        resolveErr(msgHandle);
        return;
    }
    DisplayHostError(requestHandle);


    /* Call function ReturnData() to send result to the client. */
    ReturnData(requestHandle);

    /* Delete request handle, and error handle. */
    cwbDB_DeleteSQLRequestHandle(requestHandle, msgHandle);
    cwbSV_DeleteErrHandle(msgHandle);

    return;
} /* end of SQLRequest */


